# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete a tag from a container image in Artifact Registry."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import docker_util
from googlecloudsdk.command_lib.artifacts import flags


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Delete(base.Command):
  """Delete a tag from a container image in Artifact Registry.

  A valid Docker tag has the format of

     [<location>-]docker.pkg.dev/PROJECT_ID/REPOSITORY-ID/IMAGE_PATH:tag
  """

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
  To delete tag `my-tag` from image `busy-box` in `us-west1`:

    $ {command} us-west1-docker.pkg.dev/my-project/my-repository/busy-box:my-tag
""",
  }

  @staticmethod
  def Args(parser):
    flags.GetTagRequiredArg().AddToParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.
    """
    docker_util.DeleteDockerTag(args)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for the compute global vm extension policies extensions commands."""

from googlecloudsdk.command_lib.compute import flags as compute_flags


def MakeGlobalVmExtensionPolicyArg() -> compute_flags.ResourceArgument:
  return compute_flags.ResourceArgument(
      resource_name='global vm extension policy',
      global_collection='compute.globalVmExtensionPolicies',
      required=True,
      plural=False,
  )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to add a Cloud CDN Signed URL key to a backend service."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.operations import poller
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import signed_url_flags
from googlecloudsdk.command_lib.compute.backend_services import flags
from googlecloudsdk.core.util import files


class AddSignedUrlKey(base.UpdateCommand):
  """Add Cloud CDN Signed URL key to a backend service.

  *{command}* is used to add a new Cloud CDN Signed URL key to a backend
  service.

  Cloud CDN Signed URLs give you a way to serve responses from the
  globally distributed CDN cache, even if the request needs to be
  authorized.

  Signed URLs are a mechanism to temporarily give a client access to a
  private resource without requiring additional authorization. To achieve
  this, the full request URL that should be allowed is hashed
  and cryptographically signed. By using the signed URL you give it, that
  one request will be considered authorized to receive the requested
  content.

  Generally, a signed URL can be used by anyone who has it. However, it
  is usually only intended to be used by the client that was directly
  given the URL. To mitigate this, they expire at a time chosen by the
  issuer. To minimize the risk of a signed URL being shared, it is recommended
  that the signed URL be set to expire as soon as possible.

  A 128-bit secret key is used for signing the URLs.
  """

  @staticmethod
  def Args(parser):
    """Set up arguments for this command."""
    flags.GLOBAL_BACKEND_SERVICE_ARG.AddArgument(parser)
    signed_url_flags.AddCdnSignedUrlKeyName(parser, required=True)
    signed_url_flags.AddCdnSignedUrlKeyFile(parser, required=True)

  def Run(self, args):
    """Issues the request to add Signed URL key to the backend bucket."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    api_client = holder.client.apitools_client
    messages = holder.client.messages
    service = api_client.backendServices

    backend_service_ref = flags.GLOBAL_BACKEND_SERVICE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(holder.client))
    key_value = files.ReadFileContents(args.key_file).rstrip()
    request = messages.ComputeBackendServicesAddSignedUrlKeyRequest(
        project=backend_service_ref.project,
        backendService=backend_service_ref.Name(),
        signedUrlKey=messages.SignedUrlKey(
            keyName=args.key_name, keyValue=key_value))

    operation = service.AddSignedUrlKey(request)
    operation_ref = holder.resources.Parse(
        operation.selfLink, collection='compute.globalOperations')

    operation_poller = poller.Poller(service)
    return waiter.WaitFor(operation_poller, operation_ref,
                          'Adding Cloud CDN Signed URL key to [{0}]'.format(
                              backend_service_ref.Name()))



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for GCS errors."""


from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.storage import errors as cloud_errors

ERROR_TRANSLATION = [
    (
        apitools_exceptions.HttpNotFoundError,
        None,
        cloud_errors.GcsNotFoundError,
    ),
    (
        apitools_exceptions.HttpError,
        409,
        cloud_errors.GcsConflictError,
    ),
    (
        apitools_exceptions.HttpError,
        412,
        cloud_errors.GcsPreconditionFailedError,
    ),
    (apitools_exceptions.HttpError, None, cloud_errors.GcsApiError),
]


def get_status_code(error):
  if error.response:
    return error.response.get('status')


def catch_http_error_raise_gcs_api_error(format_str=None):
  """Decorator catches HttpError and returns GcsApiError with custom message.

  Args:
    format_str (str): A googlecloudsdk.api_lib.util.exceptions.HttpErrorPayload
      format string. Note that any properties that are accessed here are on the
      HttpErrorPayload object, not the object returned from the server.

  Returns:
    A decorator that catches apitools.HttpError and returns GcsApiError with a
      customizable error message.
  """
  return cloud_errors.catch_error_raise_cloud_api_error(
      ERROR_TRANSLATION,
      format_str=format_str,
      status_code_getter=get_status_code,
  )



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for removing project-wide metadata."""


from apitools.base.py import encoding

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import metadata_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


class RemoveMetadata(base.UpdateCommand):
  """Remove project-wide metadata entries.

  *{command}* can be used to remove project-wide metadata entries.
  """

  @staticmethod
  def Args(parser):
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '--all',
        action='store_true',
        default=False,
        help='If provided, all metadata entries are removed.')
    group.add_argument(
        '--keys',
        type=arg_parsers.ArgList(min_length=1),
        metavar='KEY',
        help='The keys of the entries to remove.')

  def CreateReference(self, resources):
    return resources.Parse(
        properties.VALUES.core.project.GetOrFail(),
        collection='compute.projects')

  def GetGetRequest(self, client, project_ref):
    return (client.apitools_client.projects,
            'Get',
            client.messages.ComputeProjectsGetRequest(**project_ref.AsDict()))

  def GetSetRequest(self, client, project_ref, replacement):
    return (client.apitools_client.projects,
            'SetCommonInstanceMetadata',
            client.messages.ComputeProjectsSetCommonInstanceMetadataRequest(
                metadata=replacement.commonInstanceMetadata,
                **project_ref.AsDict()))

  def Modify(self, client, args, existing):
    new_object = encoding.JsonToMessage(
        type(existing), encoding.MessageToJson(existing))
    existing_metadata = existing.commonInstanceMetadata
    new_object.commonInstanceMetadata = metadata_utils.RemoveEntries(
        client.messages,
        existing_metadata=existing_metadata,
        keys=args.keys,
        remove_all=args.all)

    if metadata_utils.MetadataEqual(existing_metadata,
                                    new_object.commonInstanceMetadata):
      return None
    else:
      return new_object

  def Run(self, args):
    if not args.all and not args.keys:
      raise exceptions.ArgumentError(
          'One of [--all] or [--keys] must be provided.')

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    project_ref = self.CreateReference(holder.resources)
    get_request = self.GetGetRequest(client, project_ref)

    objects = client.MakeRequests([get_request])

    new_object = self.Modify(client, args, objects[0])

    # If existing object is equal to the proposed object or if
    # Modify() returns None, then there is no work to be done, so we
    # print the resource and return.
    if not new_object or objects[0] == new_object:
      log.status.Print(
          'No change requested; skipping update for [{0}].'.format(
              objects[0].name))
      return objects

    return client.MakeRequests(
        [self.GetSetRequest(client, project_ref, new_object)])



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Cloud Pub/Sub Message Transforms API."""

from googlecloudsdk.api_lib.pubsub import utils
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import exceptions


class EmptyMessageException(exceptions.Error):
  """Error when no message was specified for a Test operation."""


class EmptyFilePathException(exceptions.Error):
  """Error when no message transforms file was specified for a Validate operation."""


def GetClientInstance(no_http=False):
  return apis.GetClientInstance('pubsub', 'v1', no_http=no_http)


def GetMessagesModule(client=None):
  client = client or GetClientInstance()
  return client.MESSAGES_MODULE


class MessageTransformsClient(object):
  """Client for message transforms service in the Cloud Pub/Sub API."""

  def __init__(self, client=None, messages=None):
    self.client = client or GetClientInstance()
    self.messages = messages or GetMessagesModule(client)
    self._service = self.client.projects

  def Validate(self, project_ref, message_transform_file=None):
    """Validates a message transform.

    Args:
      project_ref (Resource): Resource reference for the project.
      message_transform_file (str): The file path to the JSON or YAML file
        containing the message transform.

    Returns:
      ValidateMessageTransformResponse (success) if the message transform is
      valid, otherwise an error.

    Raises:
      EmptyFilePathException: If no message transform file was specified.
    """
    if not message_transform_file:
      raise EmptyFilePathException(
          'You need to specify a path to JSON or YAML file containing the'
          ' message transform to validate.'
      )

    try:
      message_transform = utils.GetMessageTransformFromFileForValidation(
          self.messages.MessageTransform,
          message_transform_file,
      )
    except (
        utils.MessageTransformsInvalidFormatError,
        utils.MessageTransformsEmptyFileError,
        utils.MessageTransformsMissingFileError,
    ) as e:
      e.args = (utils.GetErrorMessage(e),)
      raise
    validate_request = self.messages.PubsubProjectsValidateMessageTransformRequest(
        project=project_ref.RelativeName(),
        validateMessageTransformRequest=self.messages.ValidateMessageTransformRequest(
            messageTransform=message_transform,
        ),
    )
    return self._service.ValidateMessageTransform(validate_request)

  def Test(
      self,
      project_ref,
      message_body=None,
      attributes=None,
      message_transforms_file=None,
      topic_ref=None,
      subscription_ref=None,
  ):
    """Tests applying message transforms to a message.

    Args:
      project_ref (Resource): Resource reference for the project.
      message_body (bytes): The message to test.
      attributes (list[AdditionalProperty]): List of attributes to attach to the
        message.
      message_transforms_file (str): The file path to the JSON or YAML file
        containing the message transforms.
      topic_ref (Resource): The topic containing the message transforms to test
        against.
      subscription_ref (Resource): The subscription containing the message
        transforms to test against.

    Returns:
      TestMessageTransformsResponse which contains a list of TransformedMessage.

    Raises:
      EmptyMessageException: If no message body or attributes were specified.
      EmptyMessageTransformsException: If no message
      transforms file/topic/subscription were specified.
    """
    if not message_body and not attributes:
      raise EmptyMessageException(
          'You cannot send an empty message. You must specify either a '
          'MESSAGE, one or more ATTRIBUTE, or both.'
      )

    message = self.messages.PubsubMessage(
        data=message_body,
        attributes=self.messages.PubsubMessage.AttributesValue(
            additionalProperties=attributes
        ),
    )
    message_transforms = None
    if message_transforms_file:
      try:
        message_transforms = utils.GetMessageTransformsFromFile(
            self.messages.MessageTransform,
            message_transforms_file,
        )
      except (
          utils.MessageTransformsInvalidFormatError,
          utils.MessageTransformsEmptyFileError,
          utils.MessageTransformsMissingFileError,
      ) as e:
        e.args = (utils.GetErrorMessage(e),)
        raise

    message_transforms_msg = (
        self.messages.MessageTransforms(messageTransforms=message_transforms)
        if message_transforms
        else None
    )

    test_request = self.messages.PubsubProjectsTestMessageTransformsRequest(
        project=project_ref.RelativeName(),
        testMessageTransformsRequest=self.messages.TestMessageTransformsRequest(
            message=message,
            messageTransforms=message_transforms_msg,
            topic=topic_ref.RelativeName() if topic_ref else None,
            subscription=subscription_ref.RelativeName()
            if subscription_ref
            else None,
        ),
    )
    return self._service.TestMessageTransforms(test_request)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing addresses."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.addresses import flags


class Describe(base.DescribeCommand):
  r"""Display detailed information about a reserved static address.

  *{command}* displays all data associated with a reserved static address in a
  project.

  ## EXAMPLES
  To get details about a global address, run:

    $ {command} my-address-name --global

  To get details about a regional address, run:

    $ {command} my-address-name --region=us-central1
  """

  ADDRESS_ARG = None

  @staticmethod
  def Args(parser):
    Describe.ADDRESS_ARG = flags.AddressArgument(plural=False)
    Describe.ADDRESS_ARG.AddArgument(parser, operation_type='describe')

  def Run(self, args):
    """Issues request necessary to describe the Addresses."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    address_ref = Describe.ADDRESS_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client))

    if address_ref.Collection() == 'compute.addresses':
      service = client.apitools_client.addresses
      request = client.messages.ComputeAddressesGetRequest(
          **address_ref.AsDict())
    elif address_ref.Collection() == 'compute.globalAddresses':
      service = client.apitools_client.globalAddresses
      request = client.messages.ComputeGlobalAddressesGetRequest(
          **address_ref.AsDict())

    return client.MakeRequests([(service, 'Get', request)])[0]



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating multi-MIGs."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.multi_migs import utils as api_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.multi_migs import utils


DETAILED_HELP = {
    'brief': 'Create a Compute Engine multi-MIG',
    'DESCRIPTION': """\
        *{command}* creates a Compute Engine multi-MIG.
    """,
    'EXAMPLES': """\
      Running:

              $ {command} example-multimig --workload-policy=example-policy

      will create a multi-MIG called 'example-multimig'
      with a workload policy called 'example-policy' in the region and project which were set by `gcloud config set`.

      You can provide full path to multi-MIG name to override the region and
      project or use `--region` and `--project` flags.

      Example:

              $ {command} projects/example-project/regions/us-central1/multiMigs/example-multimig --workload-policy=example-policy --description="Multi-MIG with workload policy"

      will create a multi-MIG called 'example-multimig'
      in the region `us-central1` and project 'example-project' with a
      workload policy called 'projects/example-project/regions/us-central1/resourcePolicies/example-policy' and a description of 'Multi-MIG with workload policy'.
    """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Create(base.CreateCommand):
  """Create a multi-MIG."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    utils.AddMultiMigNameArgToParser(
        parser, base.ReleaseTrack.ALPHA.name.lower()
    )
    parser.add_argument(
        '--description',
        help='Sets description to multi-MIG.',
    )
    parser.add_argument(
        '--workload-policy',
        help='Specifies a workload policy for multi-MIG.',
    )

  def Run(self, args):
    """Creates a multi-MIG.

    Args:
      args: the argparse arguments that this command was invoked with.

    Returns:
      List containing one resource.
    """
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    resources = holder.resources
    multi_mig_ref = args.CONCEPTS.multi_mig.Parse()
    multi_mig = client.messages.MultiMig(
        name=multi_mig_ref.Name(),
        description=args.description,
        region=multi_mig_ref.region,
        resourcePolicies=utils.MakeResourcePolicy(
            args, resources, client.messages, multi_mig_ref
        ),
    )
    return api_utils.Insert(client, multi_mig, multi_mig_ref)


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(Create):
  """Create a multi-MIG."""

  @classmethod
  def Args(cls, parser):
    utils.AddMultiMigNameArgToParser(
        parser, base.ReleaseTrack.ALPHA.name.lower()
    )
    parser.add_argument(
        '--description',
        help='Sets description to multi-MIG.',
    )
    parser.add_argument(
        '--workload-policy',
        help=(
            'Taking the resource policy ID and specifies a workload policy'
            ' for multi-MIG.'
        ),
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update a Tensorboard run in Vertex AI."""


from googlecloudsdk.api_lib.ai.tensorboard_runs import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import errors
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


def _AddArgs(parser):
  flags.AddTensorboardRunResourceArg(parser, 'to update')
  flags.GetDisplayNameArg('tensorboard run', required=False).AddToParser(parser)
  flags.GetDescriptionArg('tensorboard run').AddToParser(parser)
  labels_util.AddUpdateLabelsFlags(parser)


def _Run(args, version):
  """Update an existing Vertex AI Tensorboard run."""
  tensorboard_run_ref = args.CONCEPTS.tensorboard_run.Parse()
  args.region = tensorboard_run_ref.AsDict()['locationsId']
  with endpoint_util.AiplatformEndpointOverrides(version, region=args.region):
    try:
      op = client.TensorboardRunsClient(version=version).Patch(
          tensorboard_run_ref, args)
    except errors.NoFieldsSpecifiedError:
      available_update_args = [
          'display_name', 'update_labels', 'clear_labels', 'remove_labels',
          'description'
      ]
      if not any(args.IsSpecified(arg) for arg in available_update_args):
        raise
      log.status.Print('No update to perform.')
      return None
    else:
      log.UpdatedResource(op.name, kind='Vertex AI Tensorboard run')
      return op


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class UpdateBeta(base.UpdateCommand):
  """Update an existing Vertex AI Tensorboard run."""

  detailed_help = {
      'EXAMPLES':
          """\
          To update Tensorboard Time Run `my-tensorboard-run` in Tensorboard `12345`, Tensorboard Experiment `my-tensorboard-experiment`, with the display name `updated display name`:

              $ {command} projects/my-project/locations/us-central1/tensorboards/12345/experiments/my-tensorboard-experiment/runs/my-tensorboard-run --display-name="updated display name"

          Or with flags:

              $ {command} my-tensorboard-run --tensorboard-id=12345 --tensorboard-experiment-id=my-tensorboard-experiment --display-name="updated display name"
          """,
  }

  @staticmethod
  def Args(parser):
    _AddArgs(parser)

  def Run(self, args):
    return _Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""ai-platform versions delete command."""


from googlecloudsdk.api_lib.ml_engine import operations
from googlecloudsdk.api_lib.ml_engine import versions_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ml_engine import endpoint_util
from googlecloudsdk.command_lib.ml_engine import flags
from googlecloudsdk.command_lib.ml_engine import region_util
from googlecloudsdk.command_lib.ml_engine import versions_util


def _AddDeleteArgs(parser):
  flags.GetModelName(positional=False, required=True).AddToParser(parser)
  flags.GetRegionArg(include_global=True).AddToParser(parser)
  flags.VERSION_NAME.AddToParser(parser)


def _Run(args):
  region = region_util.GetRegion(args)
  with endpoint_util.MlEndpointOverrides(region=region):
    client = versions_api.VersionsClient()
    return versions_util.Delete(
        client, operations.OperationsClient(), args.version, model=args.model)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Delete an existing AI Platform version."""

  @staticmethod
  def Args(parser):
    _AddDeleteArgs(parser)

  def Run(self, args):
    return _Run(args)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class DeleteBeta(base.DeleteCommand):
  """Delete an existing AI Platform version."""

  @staticmethod
  def Args(parser):
    _AddDeleteArgs(parser)

  def Run(self, args):
    return _Run(args)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of gcloud Procurement consumer entitlements list command."""


from googlecloudsdk.api_lib.commerce_procurement import apis
from googlecloudsdk.calliope import base


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class List(base.ListCommand):
  """Returns the List of Entitlement objects resulting from the List API."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: argparse.ArgumentParser to register arguments with.
    """
    base.FILTER_FLAG.RemoveFromParser(parser)
    base.LIMIT_FLAG.RemoveFromParser(parser)
    base.SORT_BY_FLAG.RemoveFromParser(parser)
    base.URI_FLAG.RemoveFromParser(parser)
    parser.add_argument(
        '--page-token',
        help=('Token that specifies the next page in the list.'))

  def Run(self, args):
    """Runs the command.

    Args:
      args: The arguments that were provided to this command invocation.

    Returns:
      List of entitlements and next token if applicable.
    """
    return apis.Entitlements.List(args.page_size, args.page_token)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""bigtable tables list command."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.bigtable import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bigtable import arguments
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


def _GetUriFunction(resource):
  table_ref = resources.REGISTRY.ParseRelativeName(
      resource.name,
      collection='bigtableadmin.projects.instances.tables')
  return table_ref.SelfLink()


class ListInstances(base.ListCommand):
  """List existing Bigtable instance tables.

  ## EXAMPLES
  To list all tables in an instance, run:

    $ {command} --instances=INSTANCE_NAME

  To list all tables in several instances, run:
    $ {command} --instances=INSTANCE_NAME1,INSTANCE_NAME2
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.display_info.AddFormat("""
          table(
            name.basename():sort=1
          )
        """)
    parser.display_info.AddUriFunc(_GetUriFunction)
    arguments.ArgAdder(parser).AddInstance(
        positional=False, required=True, multiple=True)

  def Run(self, args):
    cli = util.GetAdminClient()
    msgs = util.GetAdminMessages()

    instances = args.instances
    results = []
    for instance in instances:
      instance_ref = resources.REGISTRY.Parse(
          instance,
          params={'projectsId': properties.VALUES.core.project.GetOrFail},
          collection='bigtableadmin.projects.instances')

      request = msgs.BigtableadminProjectsInstancesTablesListRequest(
          parent=instance_ref.RelativeName(),)

      for table in list_pager.YieldFromList(
          cli.projects_instances_tables,
          request,
          field='tables',
          batch_size_attribute=None):
        results.append(table)

    return results



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for setting metadata on project zonal metadata."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import instance_settings_metadata_utils as metadata_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.project_zonal_metadata import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class AddMetadata(base.UpdateCommand):
  # pylint:disable=line-too-long
  """Add or update project zonal metadata.

  *{command}* is used to add or update project zonal metadata for your VM
  instances. Project zonal metadata values propagate to all VMs within the
  specified zone. Every VM has access to a metadata server that you can use to
  query the configured project zonal metadata values. To set metadata for
  individual instances, use `gcloud compute instances add-metadata`. For
  information about metadata, see
  [](https://cloud.google.com/compute/docs/metadata).

  Only the metadata keys that you provide in the command get mutated. All other
  existing metadata entries remain the same.
  """
  # pylint:enable=line-too-long

  detailed_help = {'EXAMPLES': """\
        To set the project zonal metadata with key=value in the zone ``us-central1-a''
        for the project ``my-gcp-project'', run:

        $ {command} --metadata=key=value
        --zone=us-central1-a --project=my-gcp-project

        For more information and examples for setting project zonal metadata, see
        [](https://cloud.google.com/compute/docs/metadata/setting-custom-metadata#set-custom-project-zonal-metadata)
      """}

  @staticmethod
  def Args(parser):
    flags.ProjectZonalMetadataAddMetadataFlags(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    service = client.apitools_client.instanceSettings
    get_request = client.messages.ComputeInstanceSettingsGetRequest(
        project=properties.VALUES.core.project.GetOrFail(), zone=args.zone
    )
    existing_instance_settings = client.MakeRequests(
        [(service, 'Get', get_request)]
    )[0]
    fingerprint = existing_instance_settings.fingerprint
    if metadata_utils.IsRequestMetadataSameAsExistingMetadata(
        args.metadata,
        metadata_utils.ConstructMetadataDict(
            existing_instance_settings.metadata
        ),
    ):
      log.status.Print(
          'No change requested; skipping update for project zonal metadata in'
          ' zone [{0}].'.format(existing_instance_settings.zone)
      )
      return existing_instance_settings.metadata
    request = client.messages.ComputeInstanceSettingsPatchRequest(
        instanceSettings=client.messages.InstanceSettings(
            fingerprint=fingerprint,
            metadata=metadata_utils.ConstructInstanceSettingsMetadataMessage(
                client.messages, args.metadata
            ),
        ),
        project=properties.VALUES.core.project.GetOrFail(),
        updateMask=metadata_utils.ConstructUpdateMask(args.metadata.keys()),
        zone=args.zone,
    )
    # TODO(b/271293873):Remove no_followup=True once singleton support is added.
    return client.MakeRequests(
        [(service, 'Patch', request)],
        no_followup=True,
    )[0]



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Functions to add flags in cluster upgrade commands."""


import textwrap

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import parser_arguments


class ClusterUpgradeFlags:
  """Add flags to the cluster upgrade command surface."""

  def __init__(self, parser: parser_arguments.ArgumentInterceptor):
    self._parser = parser

  @property
  def parser(self):  # pylint: disable=invalid-name
    return self._parser

  def AddShowLinkedClusterUpgrade(self):
    """Adds the --show-linked-cluster-upgrade flag."""
    self.parser.add_argument(
        '--show-linked-cluster-upgrade',
        action='store_true',
        default=None,
        help=textwrap.dedent("""\
        Shows the cluster upgrade feature information for the current fleet as
        well as information for all other fleets linked in the same rollout
        sequence (provided that the caller has permission to view the upstream
        and downstream fleets). This displays cluster upgrade information for
        fleets in the current fleet's rollout sequence in order of furthest
        upstream to downstream.

        To view the cluster upgrade feature information for the rollout
        sequence containing the current fleet, run:

          $ {command} --show-linked-cluster-upgrade
        """),
    )

  def AddDefaultUpgradeSoakingFlag(self):
    """Adds the --default-upgrade-soaking flag."""
    self.parser.add_argument(
        '--default-upgrade-soaking',
        type=arg_parsers.Duration(),
        help=textwrap.dedent("""\
        Note: This flag only applies to Rollout Sequencing v1, not Rollout
        Sequencing v2 (which uses custom stages). If using Rollout
        Sequencing v1 and this flag is not provided, a default value of 7
        days will be used.

        Configures the default soaking duration for each upgrade propagating
        through the current fleet to become "COMPLETE". Soaking begins after
        all clusters in the fleet are on the target version, or after 30 days
        if all cluster upgrades are not complete. Once an upgrade state becomes
        "COMPLETE", it will automatically be propagated to the downstream
        fleet. Max is 30 days.

        To configure Rollout Sequencing for a fleet, this attribute must be
        set. To do this while specifying a default soaking duration of 7 days,
        run:

          $ {command} --default-upgrade-soaking=7d
        """),
    )

  def AddUpgradeSoakingOverrideFlags(self, with_destructive=False):
    if with_destructive:
      group = self.parser.add_mutually_exclusive_group()
      self._AddRemoveUpgradeSoakingOverridesFlag(group)
      self._AddUpgradeSoakingOverrideFlags(group)
    else:
      self._AddUpgradeSoakingOverrideFlags(self.parser)

  def _AddRemoveUpgradeSoakingOverridesFlag(
      self, group: parser_arguments.ArgumentInterceptor
  ):
    """Adds the --remove-upgrade-soaking-overrides flag.

    Args:
      group: The group that should contain the flag.
    """
    group.add_argument(
        '--remove-upgrade-soaking-overrides',
        action='store_true',
        default=None,
        help=textwrap.dedent("""\
        Removes soaking time overrides for all upgrades propagating through the
        current fleet. Consequently, all upgrades will follow the soak time
        configured by `--default-upgrade-soaking` until new overrides are
        configured with `--add_upgrade_soaking_override` and
        `--upgrade_selector`.

        To remove all configured soaking time overrides, run:

          $ {command} --remove-upgrade-soaking-overrides
        """),
    )

  def _AddUpgradeSoakingOverrideFlags(
      self, group: parser_arguments.ArgumentInterceptor
  ):
    """Adds upgrade soaking override flags.

    Args:
      group: The group that should contain the upgrade soaking override flags.
    """
    group = group.add_group(help=textwrap.dedent("""\
        Upgrade soaking override.

        Defines a specific soaking time override for a particular upgrade
        propagating through the current fleet that supercedes the default
        soaking duration configured by `--default-upgrade-soaking`.

        To set an upgrade soaking override of 12 hours for the upgrade with
        name, `k8s_control_plane`, and version, `1.23.1-gke.1000`, run:

          $ {command} \
              --add-upgrade-soaking-override=12h \
              --upgrade-selector=name="k8s_control_plane",version="1.23.1-gke.1000"
        """))
    self._AddAddUpgradeSoakingOverrideFlag(group)
    self._AddUpgradeSelectorFlag(group)

  def _AddAddUpgradeSoakingOverrideFlag(
      self, group: parser_arguments.ArgumentInterceptor
  ):
    """Adds the --add-upgrade-soaking-override flag.

    Args:
      group: The group that should contain the flag.
    """
    group.add_argument(
        '--add-upgrade-soaking-override',
        type=arg_parsers.Duration(),
        required=True,
        help=textwrap.dedent("""\
        Overrides the soaking time for a particular upgrade name and version
        propagating through the current fleet. Set soaking to 0 days to bypass
        soaking and fast-forward the upgrade to the downstream fleet.

        See `$ gcloud topic datetimes` for information on duration formats.
        """),
    )

  def _AddUpgradeSelectorFlag(
      self, group: parser_arguments.ArgumentInterceptor
  ):
    """Adds the --ugprade-selector flag.

    Args:
      group: The group that should contain the flag.
    """
    # TODO(b/233397411): Add link to documentation here.
    group.add_argument(
        '--upgrade-selector',
        type=UpgradeSelector(),
        required=True,
        help=textwrap.dedent("""\
        Name and version of the upgrade to be overridden where version is a
        full GKE version. Currently, name can be either `k8s_control_plane` or
        `k8s_node`.
        """),
    )

  def AddUpstreamFleetFlags(self, with_destructive=False):
    """Adds upstream fleet flags."""
    if with_destructive:
      group = self.parser.add_mutually_exclusive_group()
      self._AddUpstreamFleetFlag(group)
      self._AddResetUpstreamFleetFlag(group)
    else:
      self._AddUpstreamFleetFlag(self.parser)

  def _AddUpstreamFleetFlag(self, group: parser_arguments.ArgumentInterceptor):
    """Adds the --upstream-fleet flag.

    Args:
      group: The group that should contain the flag.
    """
    # TODO(b/233397411): Add link to documentation here.
    group.add_argument(
        '--upstream-fleet',
        type=str,
        help=textwrap.dedent("""\
        The upstream fleet. GKE will finish upgrades on the upstream fleet
        before applying the same upgrades to the current fleet.

        To configure the upstream fleet, run:

        $ {command} \
            --upstream-fleet={upstream_fleet}
        """),
    )

  def _AddResetUpstreamFleetFlag(
      self, group: parser_arguments.ArgumentInterceptor
  ):
    """Adds the --reset-upstream-fleet flag.

    Args:
      group: The group that should contain the flag.
    """
    group.add_argument(
        '--reset-upstream-fleet',
        action='store_true',
        default=None,
        help=textwrap.dedent("""\
        Clears the relationship between the current fleet and its upstream
        fleet in the rollout sequence.

        To remove the link between the current fleet and its upstream fleet,
        run:

          $ {command} --reset-upstream-fleet
        """),
    )


class UpgradeSelector(arg_parsers.ArgDict):
  """Extends the ArgDict type to properly parse --upgrade-selector argument."""

  def __init__(self):
    super(UpgradeSelector, self).__init__(
        spec={'name': str, 'version': str},
        required_keys=['name', 'version'],
        max_length=2,
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute network edge security services commands."""


from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags


class NetworkEdgeSecurityServicesCompleter(
    compute_completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(NetworkEdgeSecurityServicesCompleter, self).__init__(
        collection='compute.networkEdgeSecurityServices',
        list_command='compute network-edge-security-services list --uri',
        **kwargs)


def NetworkEdgeSecurityServiceArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='network edge security service',
      completer=NetworkEdgeSecurityServicesCompleter,
      plural=plural,
      custom_plural='network edge security services',
      required=required,
      regional_collection='compute.networkEdgeSecurityServices')



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing VPN tunnels."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import filter_rewrite
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.api_lib.compute.vpn_tunnels import vpn_tunnels_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.vpn_tunnels import flags
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_projection_spec


@base.ReleaseTracks(
    base.ReleaseTrack.BETA, base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW
)
@base.UniverseCompatible
class ListBetaGA(base.ListCommand):
  """List VPN tunnels."""

  # Placeholder to indicate that a detailed_help field exists and should
  # be set outside the class definition.
  detailed_help = None

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
    lister.AddRegionsArg(parser)
    parser.display_info.AddCacheUpdater(flags.VpnTunnelsCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    helper = vpn_tunnels_utils.VpnTunnelHelper(holder)

    project = properties.VALUES.core.project.GetOrFail()
    display_info = args.GetDisplayInfo()
    defaults = resource_projection_spec.ProjectionSpec(
        symbols=display_info.transforms, aliases=display_info.aliases)
    args.filter, filter_expr = filter_rewrite.Rewriter().Rewrite(
        args.filter, defaults=defaults)
    return helper.List(
        project=project, filter_expr=filter_expr, regions=args.regions
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListAlpha(ListBetaGA):
  """List VPN tunnels."""

  @staticmethod
  def Args(parser):
    ListBetaGA.Args(parser)
    parser.display_info.AddFormat(flags.HA_VPN_LIST_FORMAT)


ListBetaGA.detailed_help = base_classes.GetRegionalListerHelp('VPN tunnels')
ListAlpha.detailed_help = ListBetaGA.detailed_help



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for adding or updating or patching partner metadata."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import partner_metadata_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.compute.instances import flags


DETAILED_HELP = {
    'DESCRIPTION': """\
          {command} can be used to add or update or patch partner metadata of a
        virtual machine instance. Every instance has access to a
        metadata server that can be used to query partner metadata that has
        been set through this tool. For information on metadata, see link:https://cloud.google.com/compute/docs/metadata/overview[About VM metadata].

        Only namespaces keys that are provided are mutated. Existing
        Namespaces entries will remain unaffected.

        In order to retrieve partner metadata, run:

            $ gcloud compute instances describe example-instance --zone
            us-central1-a --view=FULL --format="value(partnerMetadata)"

        where example-instance is the name of the virtual machine instance
        you're querying partner metadata from.

        """,
    'EXAMPLES': """\
        To add partner metadata under namespace ``test.compute.googleapis.com''
        to instance ``TEST_INSTANCE'' run:

          $ {command} TEST_INSTANCE \\
          --partner-metadata=test.compute.googleapis.com/entries="{ \"engine\": { \"type\": V8 } }"

        To add partner metadata from a file:

          $ {command} TEST_INSTANCE \\
          --partner-metadata-from-file=examples/engine.json

        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class InstancesAddPartnerMetadata(base.UpdateCommand):
  """Add or update partner metadata."""

  @staticmethod
  def Args(parser):
    flags.INSTANCE_ARG.AddArgument(
        parser, operation_type='set partner metadata on'
    )
    partner_metadata_utils.AddPartnerMetadataArgs(parser)

  def _MakePatchPartnerMetadataRequest(self, client, instance_ref, args):
    partner_metadata_dict = partner_metadata_utils.CreatePartnerMetadataDict(
        args
    )
    partner_metadata_utils.ValidatePartnerMetadata(partner_metadata_dict)
    partner_metadata_message = (
        partner_metadata_utils.ConvertPartnerMetadataDictToMessage(
            partner_metadata_dict,
            client.messages
        )
    )
    return (
        client.apitools_client.instances,
        'PatchPartnerMetadata',
        client.messages.ComputeInstancesPatchPartnerMetadataRequest(
            partnerMetadata=client.messages.PartnerMetadata(
                partnerMetadata=partner_metadata_message
            ),
            **instance_ref.AsDict()
        ),
    )

  def Run(self, args):
    if not args.partner_metadata and not args.partner_metadata_from_file:
      raise calliope_exceptions.OneOfArgumentsRequiredException(
          ['--partner-metadata', '--partner-metadata-from-file'],
          'At least one of [--partner-metadata] or'
          ' [--partner-metadata-from-file] must be provided.',
      )
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    instance_ref = flags.INSTANCE_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client),
    )
    patch_request = self._MakePatchPartnerMetadataRequest(
        client, instance_ref, args
    )
    return client.MakeRequests([patch_request])


InstancesAddPartnerMetadata.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for removing a route policy from a Compute Engine router."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.routers import flags


@base.UniverseCompatible
class RemoveRoutePolicy(base.DeleteCommand):
  """Remove a route policy from a Compute Engine router."""
  ROUTER_ARG = None

  @classmethod
  def Args(cls, parser):
    RemoveRoutePolicy.ROUTER_ARG = flags.RouterArgument()
    RemoveRoutePolicy.ROUTER_ARG.AddArgument(parser, operation_type='delete')
    parser.add_argument(
        '--policy-name',
        help="""Name of the route policy to be removed.""",
        required=True,
    )

  def Run(self, args):
    """Issues the request necessary for removing a route policy from a Router.

    Args:
      args: contains arguments passed to the command.

    Returns:
      The result of patching the router removing the route policy.
    """
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    router_ref = RemoveRoutePolicy.ROUTER_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client),
    )

    request = (
        client.apitools_client.routers,
        'DeleteRoutePolicy',
        client.messages.ComputeRoutersDeleteRoutePolicyRequest(
            **router_ref.AsDict(), policy=args.policy_name
        ),
    )

    return client.MakeRequests([request])[0]

RemoveRoutePolicy.detailed_help = {
    'DESCRIPTION': """\
  *{command}* removes a route policy from a Compute Engine router.
  """,
    'EXAMPLES': """\
       To remove a route policy `my-policy` from a router `my-router` in region `us-central1`, run:

              $ {command} my-router --region=us-central1 --policy-name=my-policy

          """,
}



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for the container cluster related commands."""


import json

from googlecloudsdk.api_lib.edge_cloud.container import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.edge_cloud.container import admin_users
from googlecloudsdk.command_lib.edge_cloud.container import fleet
from googlecloudsdk.command_lib.edge_cloud.container import resource_args
from googlecloudsdk.command_lib.edge_cloud.container import robin
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.core import resources


def GetClusterCreateRequest(args, release_track):
  """Get cluster create request message.

  Args:
    args: comand line arguments.
    release_track: release track of the command.

  Returns:
    message obj, cluster create request message.
  """
  messages = util.GetMessagesModule(release_track)
  cluster_ref = GetClusterReference(args)
  req = messages.EdgecontainerProjectsLocationsClustersCreateRequest(
      cluster=messages.Cluster(),
      clusterId=cluster_ref.clustersId,
      parent=cluster_ref.Parent().RelativeName(),
  )
  PopulateClusterMessage(req, messages, args)
  if release_track == base.ReleaseTrack.ALPHA:
    PopulateClusterAlphaMessage(req, args)
  return req


def GetClusterUpgradeRequest(args, release_track):
  """Get cluster upgrade request message.

  Args:
    args: comand line arguments.
    release_track: release track of the command.

  Returns:
    message obj, cluster upgrade request message.
  """
  messages = util.GetMessagesModule(release_track)
  cluster_ref = GetClusterReference(args)
  upgrade_cluster_req = messages.UpgradeClusterRequest()
  upgrade_cluster_req.targetVersion = args.version
  if args.schedule.upper() != 'IMMEDIATELY':
    raise ValueError('Unsupported --schedule value: ' + args.schedule)
  upgrade_cluster_req.schedule = (
      messages.UpgradeClusterRequest.ScheduleValueValuesEnum(
          args.schedule.upper()
      )
  )
  req = messages.EdgecontainerProjectsLocationsClustersUpgradeRequest()
  req.name = cluster_ref.RelativeName()
  req.upgradeClusterRequest = upgrade_cluster_req
  return req


def PopulateClusterMessage(req, messages, args):
  """Fill the cluster message from command arguments.

  Args:
    req: create cluster request message.
    messages: message module of edgecontainer cluster.
    args: command line arguments.
  """
  # cluster service IPV4 CIDR blocks have default values.
  req.cluster.networking = messages.ClusterNetworking()
  req.cluster.networking.clusterIpv4CidrBlocks = [args.cluster_ipv4_cidr]
  req.cluster.networking.servicesIpv4CidrBlocks = [args.services_ipv4_cidr]
  if flags.FlagIsExplicitlySet(args, 'default_max_pods_per_node'):
    req.cluster.defaultMaxPodsPerNode = int(args.default_max_pods_per_node)
  if flags.FlagIsExplicitlySet(args, 'labels'):
    req.cluster.labels = messages.Cluster.LabelsValue()
    req.cluster.labels.additionalProperties = []
    for key, value in args.labels.items():
      v = messages.Cluster.LabelsValue.AdditionalProperty()
      v.key = key
      v.value = value
      req.cluster.labels.additionalProperties.append(v)
  if (
      flags.FlagIsExplicitlySet(args, 'maintenance_window_recurrence')
      or flags.FlagIsExplicitlySet(args, 'maintenance_window_start')
      or flags.FlagIsExplicitlySet(args, 'maintenance_window_end')
  ):
    req.cluster.maintenancePolicy = messages.MaintenancePolicy()
    req.cluster.maintenancePolicy.window = messages.MaintenanceWindow()
    req.cluster.maintenancePolicy.window.recurringWindow = (
        messages.RecurringTimeWindow()
    )
    if flags.FlagIsExplicitlySet(args, 'maintenance_window_recurrence'):
      req.cluster.maintenancePolicy.window.recurringWindow.recurrence = (
          args.maintenance_window_recurrence
      )
    req.cluster.maintenancePolicy.window.recurringWindow.window = (
        messages.TimeWindow()
    )
    if flags.FlagIsExplicitlySet(args, 'maintenance_window_start'):
      req.cluster.maintenancePolicy.window.recurringWindow.window.startTime = (
          args.maintenance_window_start
      )
    if flags.FlagIsExplicitlySet(args, 'maintenance_window_end'):
      req.cluster.maintenancePolicy.window.recurringWindow.window.endTime = (
          args.maintenance_window_end
      )
  if flags.FlagIsExplicitlySet(args, 'control_plane_kms_key'):
    req.cluster.controlPlaneEncryption = messages.ControlPlaneEncryption()
    req.cluster.controlPlaneEncryption.kmsKey = args.control_plane_kms_key
  if flags.FlagIsExplicitlySet(args, 'zone_storage_kms_key'):
    req.cluster.zoneStorageEncryption = messages.ZoneStorageEncryption()
    req.cluster.zoneStorageEncryption.kmsKey = args.zone_storage_kms_key
  admin_users.SetAdminUsers(messages, args, req)
  fleet.SetFleetProjectPath(GetClusterReference(args), args, req)

  if flags.FlagIsExplicitlySet(args, 'external_lb_ipv4_address_pools'):
    req.cluster.externalLoadBalancerIpv4AddressPools = (
        args.external_lb_ipv4_address_pools
    )
  if flags.FlagIsExplicitlySet(args, 'version'):
    req.cluster.targetVersion = args.version
  if flags.FlagIsExplicitlySet(args, 'release_channel'):
    req.cluster.releaseChannel = messages.Cluster.ReleaseChannelValueValuesEnum(
        args.release_channel.upper()
    )
  if (
      flags.FlagIsExplicitlySet(args, 'control_plane_node_location')
      or flags.FlagIsExplicitlySet(args, 'control_plane_node_count')
      or flags.FlagIsExplicitlySet(args, 'control_plane_machine_filter')
  ):
    # creating an LCP cluster.
    req.cluster.controlPlane = messages.ControlPlane()
    req.cluster.controlPlane.local = messages.Local()
    if flags.FlagIsExplicitlySet(args, 'control_plane_node_location'):
      req.cluster.controlPlane.local.nodeLocation = (
          args.control_plane_node_location
      )
    if flags.FlagIsExplicitlySet(args, 'control_plane_node_count'):
      req.cluster.controlPlane.local.nodeCount = int(
          args.control_plane_node_count
      )
    if flags.FlagIsExplicitlySet(args, 'control_plane_machine_filter'):
      req.cluster.controlPlane.local.machineFilter = (
          args.control_plane_machine_filter
      )
    if flags.FlagIsExplicitlySet(
        args, 'control_plane_shared_deployment_policy'
    ):
      req.cluster.controlPlane.local.sharedDeploymentPolicy = (
          messages.Local.SharedDeploymentPolicyValueValuesEnum(
              args.control_plane_shared_deployment_policy.upper()
          )
      )
  if flags.FlagIsExplicitlySet(args, 'offline_reboot_ttl'):
    if not req.cluster.survivabilityConfig:
      req.cluster.survivabilityConfig = messages.SurvivabilityConfig()
    req.cluster.survivabilityConfig.offlineRebootTtl = (
        json.dumps(args.offline_reboot_ttl) + 's'
    )
  if flags.FlagIsExplicitlySet(args, 'control_plane_node_storage_schema'):
    req.cluster.controlPlane.local.controlPlaneNodeStorageSchema = (
        args.control_plane_node_storage_schema
    )
  SetContainerRuntimeConfig(req, args, messages)
  EnableGoogleGroupAuthentication(req, args, messages)


def PopulateClusterAlphaMessage(req, args):
  """Filled the Alpha cluster message from command arguments.

  Args:
    req: create cluster request message.
    args: command line arguments.
  """
  if flags.FlagIsExplicitlySet(args, 'cluster_ipv6_cidr'):
    req.cluster.networking.clusterIpv6CidrBlocks = [args.cluster_ipv6_cidr]
  if flags.FlagIsExplicitlySet(args, 'services_ipv6_cidr'):
    req.cluster.networking.servicesIpv6CidrBlocks = [args.services_ipv6_cidr]
  if flags.FlagIsExplicitlySet(args, 'external_lb_ipv6_address_pools'):
    req.cluster.externalLoadBalancerIpv6AddressPools = (
        args.external_lb_ipv6_address_pools
    )
  resource_args.SetSystemAddonsConfig(args, req)
  resource_args.SetExternalLoadBalancerAddressPoolsConfig(args, req)
  EnableClusterIsolationConfig(req, args)
  EnableRemoteBackupConfig(req, args)
  if flags.FlagIsExplicitlySet(args, 'enable_robin_cns'):
    robin.EnableRobinCNSInRequest(req, args)

  messages = util.GetMessagesModule(base.ReleaseTrack.ALPHA)
  if flags.FlagIsExplicitlySet(
      args, 'control_plane_node_system_partition_size_gib'
  ):
    SetControlPlaneNodeSystemPartitionSize(req, args, messages)


def IsLCPCluster(args):
  """Identify if the command is creating LCP cluster.

  Args:
    args: command line arguments.

  Returns:
    Boolean, indication of LCP cluster.
  """
  if (
      flags.FlagIsExplicitlySet(args, 'control_plane_node_location')
      and flags.FlagIsExplicitlySet(args, 'control_plane_node_count')
      and (
          flags.FlagIsExplicitlySet(args, 'external_lb_ipv4_address_pools')
          or flags.FlagIsExplicitlySet(args, 'external_lb_address_pools')
      )
  ):
    return True
  return False


def IsOfflineCredential(args):
  """Identify if the command is requesting an offline credential for LCP cluster.

  Args:
    args: command line arguments.

  Returns:
    Boolean, indication of requesting offline credential.
  """
  if flags.FlagIsExplicitlySet(args, 'offline_credential'):
    return True
  return False


def GetClusterReference(args):
  """Get edgecontainer cluster resources.

  Args:
    args: command line arguments.

  Returns:
    edgecontainer cluster resources.
  """
  return resources.REGISTRY.ParseRelativeName(
      args.CONCEPTS.cluster.Parse().RelativeName(),
      collection='edgecontainer.projects.locations.clusters',
  )


def ValidateClusterCreateRequest(req, release_track):
  """Validate cluster create request message.

  Args:
    req: Create cluster request message.
    release_track: Release track of the command.

  Returns:
    Single string of error message.
  """
  messages = util.GetMessagesModule(release_track)
  if (
      req.cluster.releaseChannel
      == messages.Cluster.ReleaseChannelValueValuesEnum.REGULAR
      and req.cluster.targetVersion is not None
  ):
    return (
        'Invalid Argument: REGULAR release channel does not support'
        ' specification of version'
    )
  return None


def SetContainerRuntimeConfig(req, args, messages):
  """Set container runtime config in the cluster request message.

  Args:
    req: Create cluster request message.
    args: Command line arguments.
    messages: Message module of edgecontainer cluster.
  """
  if flags.FlagIsExplicitlySet(args, 'container_default_runtime_class'):
    req.cluster.containerRuntimeConfig = messages.ContainerRuntimeConfig()
    if args.container_default_runtime_class.upper() == 'GVISOR':
      req.cluster.containerRuntimeConfig.defaultContainerRuntime = (
          messages.ContainerRuntimeConfig.DefaultContainerRuntimeValueValuesEnum.GVISOR
      )
    elif args.container_default_runtime_class.upper() == 'RUNC':
      req.cluster.containerRuntimeConfig.defaultContainerRuntime = (
          messages.ContainerRuntimeConfig.DefaultContainerRuntimeValueValuesEnum.RUNC
      )
    else:
      raise ValueError(
          'Unsupported --container_default_runtime_class value: '
          + args.container_default_runtime_class
      )


def EnableClusterIsolationConfig(req, args):
  """Set secure cluster isolation config in the cluster request message.

  Args:
   req: Create cluster request message.
   args: Command line arguments.
  """

  if flags.FlagIsExplicitlySet(args, 'enable_cluster_isolation'):
    if args.enable_cluster_isolation.upper() == 'TRUE':
      req.cluster.enableClusterIsolation = True
    elif args.enable_cluster_isolation.upper() == 'FALSE':
      req.cluster.enableClusterIsolation = False
    else:
      raise ValueError(
          'Unsupported --enable_cluster_isolation value: '
          + args.enable_cluster_isolation
      )


def EnableGoogleGroupAuthentication(req, args, messages):
  """Set Google Group authentication config in the cluster request message.

  Args:
   req: Create cluster request message.
   args: Command line arguments.
   messages: Message module of edgecontainer cluster.
  """

  if flags.FlagIsExplicitlySet(args, 'enable_google_group_authentication'):
    req.cluster.googleGroupAuthentication = (
        messages.GoogleGroupAuthenticationConfig()
    )
    req.cluster.googleGroupAuthentication.enable = (
        args.enable_google_group_authentication)


def EnableRemoteBackupConfig(req, args):
  """Set remote backup config in the cluster request message.

  Args:
   req: Create cluster request message.
   args: Command line arguments.
  """

  if flags.FlagIsExplicitlySet(args, 'enable_remote_backup'):
    req.cluster.enableRemoteBackup = args.enable_remote_backup


def SetControlPlaneNodeSystemPartitionSize(req, args, messages):
  """Set control plane node system partition size in the cluster request message.

  Args:
   req: Create cluster request message.
   args: Command line arguments.
   messages: Message module of edgecontainer cluster.
  """
  if args.control_plane_node_system_partition_size_gib == 100:
    req.cluster.controlPlane.local.controlPlaneNodeSystemPartitionSize = (
        messages.Local.ControlPlaneNodeSystemPartitionSizeValueValuesEnum.SYSTEM_PARTITION_GIB_SIZE100
    )
  elif args.control_plane_node_system_partition_size_gib == 300:
    req.cluster.controlPlane.local.controlPlaneNodeSystemPartitionSize = (
        messages.Local.ControlPlaneNodeSystemPartitionSizeValueValuesEnum.SYSTEM_PARTITION_GIB_SIZE300
    )
  else:
    raise ValueError(
        'Unsupported --control_plane_node_system_partition_size_gib value: '
        + args.control_plane_node_system_partition_size_gib
        + '; valid values are 100 and 300.'
    )



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for resetting an instance."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.instances import flags

DETAILED_HELP = {
    'brief': 'Reset a virtual machine instance.',
    'DESCRIPTION':
        """\
          *{command}* is used to perform a hard reset on a
        Compute Engine virtual machine.

        This will not perform a clean shutdown of the guest OS on the instance.
        """,
    'EXAMPLES':
        """\
        To reset an instance named ``test-instance'', run:

          $ {command} test-instance
        """
}


class Reset(base.SilentCommand):
  """Reset a virtual machine instance."""

  @staticmethod
  def Args(parser):
    flags.INSTANCES_ARG.AddArgument(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    instance_refs = flags.INSTANCES_ARG.ResolveAsResource(
        args, holder.resources,
        scope_lister=flags.GetInstanceZoneScopeLister(client))
    request_list = []
    for instance_ref in instance_refs:
      request = client.messages.ComputeInstancesResetRequest(
          instance=instance_ref.Name(),
          project=instance_ref.project,
          zone=instance_ref.zone)

      request_list.append((client.apitools_client.instances, 'Reset', request))
    return client.MakeRequests(request_list)


Reset.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to describe snapshot recycle bin policy."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
@base.DefaultUniverseOnly
class Describe(base.DescribeCommand):
  """Describe the project's or organization's snapshot recycle bin policy."""

  detailed_help = {
      'EXAMPLES': """
      To describe the snapshot recycle bin policy for project `my-project`:

        $ {command} --project=my-project

      To describe the snapshot recycle bin policy for organization `123456789`:

        $ {command} --organization=123456789
      """,
  }

  @classmethod
  def Args(cls, parser):
    project_or_organization = parser.add_mutually_exclusive_group()
    project_or_organization.add_argument(
        '--organization',
        help='Organization ID for the policy.',
    )
    project_or_organization.add_argument(
        '--project',
        help='Project ID for the policy.',
    )

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    messages = holder.client.messages

    if args.organization:
      request = messages.ComputeOrganizationSnapshotRecycleBinPolicyGetRequest(
          organization='organizations/' + args.organization
      )
      service = client.apitools_client.organizationSnapshotRecycleBinPolicy
    else:
      project = args.project or properties.VALUES.core.project.Get(
          required=True
      )
      request = messages.ComputeSnapshotRecycleBinPolicyGetRequest(
          project=project
      )
      service = client.apitools_client.snapshotRecycleBinPolicy

    return client.MakeRequests([(service, 'Get', request)])[0]



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to get IAM policy for a resource."""

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.backend_buckets import backend_buckets_utils
from googlecloudsdk.command_lib.compute.backend_buckets import flags


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.DefaultUniverseOnly
class GetIamPolicy(base.ListCommand):
  """Get the IAM policy for a Compute Engine backend bucket."""

  @staticmethod
  def Args(parser):
    flags.GLOBAL_REGIONAL_BACKEND_BUCKET_ARG_IAM.AddArgument(parser)
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client
    backend_bucket_ref = (
        flags.GLOBAL_REGIONAL_BACKEND_BUCKET_ARG_IAM.ResolveAsResource(
            args,
            holder.resources,
            default_scope=compute_scope.ScopeEnum.GLOBAL,
            scope_lister=compute_flags.GetDefaultScopeLister(holder.client),
        )
    )

    return backend_buckets_utils.GetIamPolicy(backend_bucket_ref, client)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
@base.DefaultUniverseOnly
class GetIamPolicyBeta(GetIamPolicy):
  pass


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.DefaultUniverseOnly
class GetIamPolicyAlpha(GetIamPolicyBeta):
  pass


GetIamPolicy.detailed_help = {
    'brief': 'Get the IAM policy for a Compute Engine backend bucket.',
    'DESCRIPTION': (
        """\

      *{command}* displays the IAM policy associated with a
    Compute Engine backend bucket in a project. If formatted as JSON,
    the output can be edited and used as a policy file for
    set-iam-policy. The output includes an "etag" field
    identifying the version emitted and allowing detection of
    concurrent policy updates; see
    $ {parent} set-iam-policy for additional details.  """
    ),
    'EXAMPLES': (
        """\
    To print the IAM policy for a given regional backend bucket, run:

      $ {command} my-backend-bucket --region=REGION

    To print the IAM policy for a given global backend bucket, run either of
    the following:

      $ {command} my-backend-bucket --global

      $ {command} my-backend-bucket
      """
    ),
    'API REFERENCE': (
        """\
        This command uses the compute API. The full documentation for this
    API can be found at: https://cloud.google.com/compute/"""
    ),
}



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of command for deleting folders."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import errors_util
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.command_lib.storage import folder_util
from googlecloudsdk.command_lib.storage import name_expansion
from googlecloudsdk.command_lib.storage import rm_command_util
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.command_lib.storage.tasks import task_graph_executor


@base.DefaultUniverseOnly
class Delete(base.Command):
  """Delete folders."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': """Delete folders.""",
      'EXAMPLES': """
      The following command deletes a folder named `folder`
      in a hierarchical namesapce bucket
      called `my-bucket`:

        $ {command} gs://my-bucket/folder/
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'urls',
        type=str,
        nargs='+',
        help='The URLs of the folders to delete.',
    )
    flags.add_additional_headers_flag(parser)

  def Run(self, args):
    for url_string in args.urls:
      url = storage_url.storage_url_from_string(url_string)
      errors_util.raise_error_if_not_gcs_folder_type(
          args.command_path, url, 'folder'
      )

    folder_expansion_iterator = name_expansion.NameExpansionIterator(
        args.urls,
        folder_setting=folder_util.FolderSetting.LIST_WITHOUT_OBJECTS,
        raise_error_for_unmatched_urls=True,
    )
    self.exit_code = rm_command_util.remove_folders(
        folder_expansion_iterator,
        task_status_queue=task_graph_executor.multiprocessing_context.Queue(),
        raise_error_for_unmatched_urls=True,
    )



"""Generated client library for modelarmor version v1alpha."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.modelarmor.v1alpha import modelarmor_v1alpha_messages as messages


class ModelarmorV1alpha(base_api.BaseApiClient):
  """Generated client library for service modelarmor version v1alpha."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://modelarmor.us.rep.googleapis.com/'
  MTLS_BASE_URL = 'https://modelarmor.us.rep.mtls.googleapis.com/'

  _PACKAGE = 'modelarmor'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1alpha'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'ModelarmorV1alpha'
  _URL_VERSION = 'v1alpha'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new modelarmor handle."""
    url = url or self.BASE_URL
    super(ModelarmorV1alpha, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_locations = self.FoldersLocationsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_locations = self.OrganizationsLocationsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_locations_templates = self.ProjectsLocationsTemplatesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersLocationsService(base_api.BaseApiService):
    """Service class for the folders_locations resource."""

    _NAME = 'folders_locations'

    def __init__(self, client):
      super(ModelarmorV1alpha.FoldersLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetFloorSetting(self, request, global_params=None):
      r"""Gets details of a single floor setting of a project.

      Args:
        request: (ModelarmorFoldersLocationsGetFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('GetFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/folders/{foldersId}/locations/{locationsId}/floorSetting',
        http_method='GET',
        method_id='modelarmor.folders.locations.getFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorFoldersLocationsGetFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

    def UpdateFloorSetting(self, request, global_params=None):
      r"""Updates the parameters of a single floor setting of a project.

      Args:
        request: (ModelarmorFoldersLocationsUpdateFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('UpdateFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/folders/{foldersId}/locations/{locationsId}/floorSetting',
        http_method='PATCH',
        method_id='modelarmor.folders.locations.updateFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='floorSetting',
        request_type_name='ModelarmorFoldersLocationsUpdateFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(ModelarmorV1alpha.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsService(base_api.BaseApiService):
    """Service class for the organizations_locations resource."""

    _NAME = 'organizations_locations'

    def __init__(self, client):
      super(ModelarmorV1alpha.OrganizationsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def GetFloorSetting(self, request, global_params=None):
      r"""Gets details of a single floor setting of a project.

      Args:
        request: (ModelarmorOrganizationsLocationsGetFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('GetFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/organizations/{organizationsId}/locations/{locationsId}/floorSetting',
        http_method='GET',
        method_id='modelarmor.organizations.locations.getFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorOrganizationsLocationsGetFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

    def UpdateFloorSetting(self, request, global_params=None):
      r"""Updates the parameters of a single floor setting of a project.

      Args:
        request: (ModelarmorOrganizationsLocationsUpdateFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('UpdateFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/organizations/{organizationsId}/locations/{locationsId}/floorSetting',
        http_method='PATCH',
        method_id='modelarmor.organizations.locations.updateFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='floorSetting',
        request_type_name='ModelarmorOrganizationsLocationsUpdateFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(ModelarmorV1alpha.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsTemplatesService(base_api.BaseApiService):
    """Service class for the projects_locations_templates resource."""

    _NAME = 'projects_locations_templates'

    def __init__(self, client):
      super(ModelarmorV1alpha.ProjectsLocationsTemplatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new Template in a given project and location.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Template) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates',
        http_method='POST',
        method_id='modelarmor.projects.locations.templates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId', 'templateId'],
        relative_path='v1alpha/{+parent}/templates',
        request_field='template',
        request_type_name='ModelarmorProjectsLocationsTemplatesCreateRequest',
        response_type_name='Template',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single Template.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates/{templatesId}',
        http_method='DELETE',
        method_id='modelarmor.projects.locations.templates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsTemplatesDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Template.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Template) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates/{templatesId}',
        http_method='GET',
        method_id='modelarmor.projects.locations.templates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsTemplatesGetRequest',
        response_type_name='Template',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Templates in a given project and location.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListTemplatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates',
        http_method='GET',
        method_id='modelarmor.projects.locations.templates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+parent}/templates',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsTemplatesListRequest',
        response_type_name='ListTemplatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the parameters of a single Template.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Template) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates/{templatesId}',
        http_method='PATCH',
        method_id='modelarmor.projects.locations.templates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId', 'updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='template',
        request_type_name='ModelarmorProjectsLocationsTemplatesPatchRequest',
        response_type_name='Template',
        supports_download=False,
    )

    def SanitizeModelResponse(self, request, global_params=None):
      r"""Sanitizes Model Response.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesSanitizeModelResponseRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SanitizeModelResponseResponse) The response message.
      """
      config = self.GetMethodConfig('SanitizeModelResponse')
      return self._RunMethod(
          config, request, global_params=global_params)

    SanitizeModelResponse.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates/{templatesId}:sanitizeModelResponse',
        http_method='POST',
        method_id='modelarmor.projects.locations.templates.sanitizeModelResponse',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:sanitizeModelResponse',
        request_field='sanitizeModelResponseRequest',
        request_type_name='ModelarmorProjectsLocationsTemplatesSanitizeModelResponseRequest',
        response_type_name='SanitizeModelResponseResponse',
        supports_download=False,
    )

    def SanitizeUserPrompt(self, request, global_params=None):
      r"""Sanitizes User Prompt.

      Args:
        request: (ModelarmorProjectsLocationsTemplatesSanitizeUserPromptRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (SanitizeUserPromptResponse) The response message.
      """
      config = self.GetMethodConfig('SanitizeUserPrompt')
      return self._RunMethod(
          config, request, global_params=global_params)

    SanitizeUserPrompt.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/templates/{templatesId}:sanitizeUserPrompt',
        http_method='POST',
        method_id='modelarmor.projects.locations.templates.sanitizeUserPrompt',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}:sanitizeUserPrompt',
        request_field='sanitizeUserPromptRequest',
        request_type_name='ModelarmorProjectsLocationsTemplatesSanitizeUserPromptRequest',
        response_type_name='SanitizeUserPromptResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(ModelarmorV1alpha.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (ModelarmorProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='modelarmor.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def GetFloorSetting(self, request, global_params=None):
      r"""Gets details of a single floor setting of a project.

      Args:
        request: (ModelarmorProjectsLocationsGetFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('GetFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/floorSetting',
        http_method='GET',
        method_id='modelarmor.projects.locations.getFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1alpha/{+name}',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsGetFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the [ListLocationsRequest.name] field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (ModelarmorProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations',
        http_method='GET',
        method_id='modelarmor.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v1alpha/{+name}/locations',
        request_field='',
        request_type_name='ModelarmorProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

    def UpdateFloorSetting(self, request, global_params=None):
      r"""Updates the parameters of a single floor setting of a project.

      Args:
        request: (ModelarmorProjectsLocationsUpdateFloorSettingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FloorSetting) The response message.
      """
      config = self.GetMethodConfig('UpdateFloorSetting')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateFloorSetting.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1alpha/projects/{projectsId}/locations/{locationsId}/floorSetting',
        http_method='PATCH',
        method_id='modelarmor.projects.locations.updateFloorSetting',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1alpha/{+name}',
        request_field='floorSetting',
        request_type_name='ModelarmorProjectsLocationsUpdateFloorSettingRequest',
        response_type_name='FloorSetting',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(ModelarmorV1alpha.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for creating/parsing update argument groups."""


import abc
import enum

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import arg_parsers_usage_text as usage_text
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import util as format_util
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.apis import yaml_command_schema_util as util
import six


# TODO(b/280653078) The UX is still under review. These utilities are
# liable to change and should not be used in new surface yet.

# TODO(b/283949482): Place this file in util/args and replace the duplicate
# logic in the util files.


class Prefix(enum.Enum):
  ADD = 'add'
  UPDATE = 'update'
  REMOVE = 'remove'
  CLEAR = 'clear'


class _ConvertValueType(usage_text.DefaultArgTypeWrapper):
  """Wraps flag types in arg_utils.ConvertValue while maintaining help text.

  Attributes:
    arg_gen: UpdateBasicArgumentGenerator, update argument generator
  """

  def __init__(self, arg_gen):
    super(_ConvertValueType, self).__init__(arg_gen.flag_type)
    self.field = arg_gen.field
    self.repeated = arg_gen.repeated
    self.processor = arg_gen.processor
    self._choices = arg_gen.choices

  @property
  def choices(self):
    """Returns a map of choice values to choice help text.

    Used in help text generation and not in actual input parsing.
    """
    if self._choices:
      return {choice.arg_value: choice.help_text for choice in self._choices}
    else:
      return None

  def __call__(self, arg_value):
    """Converts arg_value into type arg_type."""
    value = self.arg_type(arg_value)
    return arg_utils.ConvertValue(
        self.field,
        value,
        repeated=self.repeated,
        processor=self.processor,
        choices=util.Choice.ToChoiceMap(self._choices),
    )


class UpdateArgumentGenerator(six.with_metaclass(abc.ABCMeta, object)):
  """Update flag generator.

  To use this base class, provide required methods for parsing
  (GetArgFromNamespace and GetFieldValueFromNamespace) and override
  the flags that are needed to update the value. For example, if argument
  group requires a set flag, we would override the `set_arg` property and
  ApplySetFlag method.
  """

  def _GetTextFormatOfEmptyValue(self, value):
    if value:
      return value

    if isinstance(value, dict):
      return 'empty map'
    if isinstance(value, list):
      return 'empty list'
    if value is None:
      return 'null'

    return value

  def _CreateFlag(
      self, arg_name, flag_prefix=None, flag_type=None, action=None,
      metavar=None, help_text=None
  ):
    """Creates a flag.

    Args:
      arg_name: str, root name of the arg
      flag_prefix: Prefix | None, prefix for the flag name
      flag_type: func, type that flag is used to convert user input
      action: str, flag action
      metavar: str, user specified metavar for flag
      help_text: str, flag help text

    Returns:
      base.Argument with correct params
    """
    flag_name = arg_utils.GetFlagName(
        arg_name, flag_prefix and flag_prefix.value)
    arg = base.Argument(flag_name, action=action, help=help_text)

    if action == 'store_true':
      return arg

    arg.kwargs['type'] = flag_type
    if flag_metavar := arg_utils.GetMetavar(metavar, flag_type, flag_name):
      arg.kwargs['metavar'] = flag_metavar
    return arg

  # DEFAULT FLAGS GENERATED
  @property
  def set_arg(self):
    """Flag that sets field to specifed value."""
    return None

  @property
  def clear_arg(self):
    """Flag that clears field."""
    return None

  @property
  def update_arg(self):
    """Flag that updates value if part of existing field."""
    return None

  @property
  def remove_arg(self):
    """Flag that removes value if part of existing field."""
    return None

  def Generate(self, additional_flags=None):
    """Returns ArgumentGroup with all flags specified in generator.

    ArgumentGroup is returned where the set flag is mutually exclusive with
    the rest of the update flags. In addition, remove and clear flags are
    mutually exclusive. The following combinations are allowed

    # sets the foo value to value1,value2
    {command} --foo=value1,value2

    # adds values value3
    {command} --add-foo=value3

    # clears values and sets foo to value4,value5
    {command} --add-foo=value4,value5 --clear

    # removes value4 and adds value6
    {command} --add-foo=value6 --remove-foo=value4

    # removes value6 and then re-adds it
    {command} --add-foo=value6 --remove-foo=value6

    Args:
      additional_flags: [base.Argument], list of additional arguments needed
        to udpate the value

    Returns:
      base.ArgumentGroup, argument group containing flags
    """
    base_group = base.ArgumentGroup(
        mutex=True,
        required=False,
        hidden=self.is_hidden,
        help='Update {}.'.format(self.arg_name),
    )
    if self.set_arg:
      base_group.AddArgument(self.set_arg)

    update_group = base.ArgumentGroup(required=False)
    if self.update_arg:
      update_group.AddArgument(self.update_arg)

    clear_group = base.ArgumentGroup(mutex=True, required=False)
    if self.clear_arg:
      clear_group.AddArgument(self.clear_arg)
    if self.remove_arg:
      clear_group.AddArgument(self.remove_arg)

    if clear_group.arguments:
      update_group.AddArgument(clear_group)
    if update_group.arguments:
      base_group.AddArgument(update_group)

    if not additional_flags:
      return base_group

    wrapper_group = base.ArgumentGroup(
        required=False,
        hidden=self.is_hidden,
        help='All arguments needed to update {}.'.format(self.arg_name),
    )
    wrapper_group.AddArgument(base_group)
    for arg in additional_flags:
      wrapper_group.AddArgument(arg)
    return wrapper_group

  # METHODS REQUIRED FOR PARSING NEW VALUE
  @abc.abstractmethod
  def GetArgFromNamespace(self, namespace, arg):
    """Retrieves namespace value associated with flag.

    Args:
      namespace: The parsed command line argument namespace.
      arg: base.Argument, used to get namespace value

    Returns:
      value parsed from namespace
    """
    pass

  @abc.abstractmethod
  def GetFieldValueFromMessage(self, existing_message):
    """Retrieves existing field from message.

    Args:
      existing_message: apitools message we need to get field value from

    Returns:
      field value from apitools message
    """
    pass

  # DEFAULTED METHODS FOR PARSING NEW VALUE
  def ApplySetFlag(self, existing_val, unused_set_val):
    """Updates result to new value (No-op: implementation in subclass)."""
    return existing_val

  def ApplyClearFlag(self, existing_val, unused_clear_flag):
    """Clears existing value (No-op: implementation in subclass)."""
    return existing_val

  def ApplyRemoveFlag(self, existing_val, unused_remove_val):
    """Removes existing value (No-op: implementation in subclass)."""
    return existing_val

  def ApplyUpdateFlag(self, existing_val, unused_update_val):
    """Updates existing value (No-op: implementation in subclass)."""
    return existing_val

  def Parse(self, namespace, existing_message):
    """Parses update flags from namespace and returns updated message field.

    Args:
      namespace: The parsed command line argument namespace.
      existing_message: Apitools message that exists for given resource.

    Returns:
      Modified existing apitools message field.
    """
    result = self.GetFieldValueFromMessage(existing_message)
    set_value, clear_value, remove_value, update_value = (
        self.GetArgFromNamespace(namespace, self.set_arg),
        self.GetArgFromNamespace(namespace, self.clear_arg),
        self.GetArgFromNamespace(namespace, self.remove_arg),
        self.GetArgFromNamespace(namespace, self.update_arg),
    )

    # Whether or not the flags are mutually exclusive are determined by the
    # ArgumentGroup generated. We do not want to duplicate the mutex logic
    # so instead we consistently apply all flags in same order, first by
    # removing and then adding values.

    # Remove values
    result = self.ApplyClearFlag(result, clear_value)
    result = self.ApplyRemoveFlag(result, remove_value)

    # Add values
    result = self.ApplySetFlag(result, set_value)
    result = self.ApplyUpdateFlag(result, update_value)

    return result


class UpdateBasicArgumentGenerator(UpdateArgumentGenerator):
  """Update flag generator for simple flags."""

  @classmethod
  def FromArgData(cls, arg_data, field):
    """Creates a flag generator from yaml arg data and request message.

    Args:
      arg_data: yaml_arg_schema.Argument, data about flag being generated
      field: messages.Field, apitools field instance.

    Returns:
      UpdateArgumentGenerator, the correct version of flag generator
    """
    flag_type, action = arg_utils.GenerateFlagType(field, arg_data)

    is_repeated = (
        field.repeated if arg_data.repeated is None else arg_data.repeated
    )
    field_type = arg_utils.GetFieldType(field)

    if field_type == arg_utils.FieldType.MAP:
      gen_cls = UpdateMapArgumentGenerator
    elif is_repeated:
      gen_cls = UpdateListArgumentGenerator
    else:
      gen_cls = UpdateDefaultArgumentGenerator

    return gen_cls(
        arg_name=arg_data.arg_name,
        flag_type=flag_type,
        field=field,
        action=action,
        is_hidden=arg_data.hidden,
        help_text=arg_data.help_text,
        api_field=arg_data.api_field,
        repeated=arg_data.repeated,
        processor=arg_data.processor,
        choices=arg_data.choices,
        metavar=arg_data.metavar,
    )

  def __init__(
      self,
      arg_name,
      flag_type=None,
      field=None,
      action=None,
      is_hidden=False,
      help_text=None,
      api_field=None,
      repeated=False,
      processor=None,
      choices=None,
      metavar=None,
  ):
    super(UpdateBasicArgumentGenerator, self).__init__()
    self.arg_name = format_util.NormalizeFormat(arg_name)
    self.field = field
    self.flag_type = flag_type
    self.action = action
    self.is_hidden = is_hidden
    self.help_text = help_text
    self.api_field = api_field
    self.repeated = repeated
    self.processor = processor
    self.choices = choices
    self.metavar = metavar

  def GetArgFromNamespace(self, namespace, arg):
    if arg is None:
      return None
    return arg_utils.GetFromNamespace(namespace, arg.name)

  def GetFieldValueFromMessage(self, existing_message):
    """Retrieves existing field from message."""
    if existing_message:
      existing_value = arg_utils.GetFieldValueFromMessage(
          existing_message, self.api_field
      )
    else:
      existing_value = None

    if isinstance(existing_value, list):
      existing_value = existing_value.copy()
    return existing_value

  def _CreateBasicFlag(self, **kwargs):
    return self._CreateFlag(arg_name=self.arg_name, **kwargs)


class UpdateDefaultArgumentGenerator(UpdateBasicArgumentGenerator):
  """Update flag generator for simple values."""

  @property
  def _empty_value(self):
    return None

  @property
  def set_arg(self):
    return self._CreateBasicFlag(
        flag_type=_ConvertValueType(self),
        action=self.action,
        metavar=self.metavar,
        help_text='Set {} to new value.'.format(self.arg_name),
    )

  @property
  def clear_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.CLEAR,
        action='store_true',
        help_text='Clear {} value and set to {}.'.format(
            self.arg_name, self._GetTextFormatOfEmptyValue(self._empty_value)),
    )

  def ApplySetFlag(self, existing_val, set_val):
    if set_val is not None:
      return set_val
    return existing_val

  def ApplyClearFlag(self, existing_val, clear_flag):
    if clear_flag:
      return self._empty_value
    return existing_val


class UpdateListArgumentGenerator(UpdateBasicArgumentGenerator):
  """Update flag generator for list."""

  @property
  def _empty_value(self):
    return []

  @property
  def set_arg(self):
    return self._CreateBasicFlag(
        flag_type=_ConvertValueType(self),
        action=self.action,
        metavar=self.metavar,
        help_text='Set {} to new value.'.format(self.arg_name),
    )

  @property
  def clear_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.CLEAR,
        action='store_true',
        help_text='Clear {} value and set to {}.'.format(
            self.arg_name, self._GetTextFormatOfEmptyValue(self._empty_value)),
    )

  @property
  def update_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.ADD,
        flag_type=_ConvertValueType(self),
        action=self.action,
        help_text='Add new value to {} list.'.format(self.arg_name),
    )

  @property
  def remove_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.REMOVE,
        flag_type=_ConvertValueType(self),
        action=self.action,
        help_text='Remove existing value from {} list.'.format(self.arg_name),
    )

  def _ContainsVal(self, new_val, all_vals):
    if isinstance(self.flag_type, util.EquitableType):
      return any(
          self.flag_type.Matches(new_val, val) for val in all_vals)
    else:
      return new_val in all_vals

  def ApplySetFlag(self, existing_val, set_val):
    if set_val is not None:
      return set_val
    return existing_val

  def ApplyClearFlag(self, existing_val, clear_flag):
    if clear_flag:
      return self._empty_value
    return existing_val

  def ApplyRemoveFlag(self, existing_val, remove_val):
    if remove_val is not None:
      return [
          x for x in existing_val if not self._ContainsVal(x, remove_val)]
    return existing_val

  def ApplyUpdateFlag(self, existing_val, update_val):
    if update_val is not None:
      new_vals = [
          x for x in update_val if not self._ContainsVal(x, existing_val)]
      return existing_val + new_vals
    return existing_val


class UpdateMapArgumentGenerator(UpdateBasicArgumentGenerator):
  """Update flag generator for key-value pairs ie proto map fields."""

  @property
  def _empty_value(self):
    return {}

  @property
  def _is_list_field(self):
    return self.field.name == arg_utils.ADDITIONAL_PROPS

  def _WrapOutput(self, output_list):
    """Wraps field AdditionalProperties in apitools message if needed.

    Args:
      output_list: list of apitools AdditionalProperties messages.

    Returns:
      apitools message instance.
    """
    if self._is_list_field:
      return output_list
    message = self.field.type()
    arg_utils.SetFieldInMessage(
        message, arg_utils.ADDITIONAL_PROPS, output_list)
    return message

  def _GetPropsFieldValue(self, field):
    """Retrieves AdditionalProperties field value.

    Args:
      field: apitools instance that contains AdditionalProperties field

    Returns:
      list of apitools AdditionalProperties messages.
    """
    if not field:
      return []
    if self._is_list_field:
      return field
    return arg_utils.GetFieldValueFromMessage(field, arg_utils.ADDITIONAL_PROPS)

  @property
  def set_arg(self):
    return self._CreateBasicFlag(
        flag_type=_ConvertValueType(self),
        action=self.action,
        metavar=self.metavar,
        help_text='Set {} to new value.'.format(self.arg_name),
    )

  @property
  def clear_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.CLEAR,
        action='store_true',
        help_text='Clear {} value and set to {}.'.format(
            self.arg_name, self._GetTextFormatOfEmptyValue(self._empty_value)),
    )

  @property
  def update_arg(self):
    return self._CreateBasicFlag(
        flag_prefix=Prefix.UPDATE,
        flag_type=_ConvertValueType(self),
        action=self.action,
        help_text='Update {} value or add key value pair.'.format(
            self.arg_name
        ),
    )

  @property
  def remove_arg(self):
    if self._is_list_field:
      field = self.field
    else:
      field = arg_utils.GetFieldFromMessage(
          self.field.type, arg_utils.ADDITIONAL_PROPS
      )

    key_field = arg_utils.GetFieldFromMessage(field.type, 'key')
    key_type = key_field.type or arg_utils.TYPES.get(key_field.variant)
    key_list = arg_parsers.ArgObject(
        value_type=key_type, repeated=True)

    return self._CreateBasicFlag(
        flag_prefix=Prefix.REMOVE,
        flag_type=key_list,
        action='store',
        help_text='Remove existing value from map {}.'.format(self.arg_name),
    )

  def ApplySetFlag(self, existing_val, set_val):
    if set_val is not None:
      return set_val
    return existing_val

  def ApplyClearFlag(self, existing_val, clear_flag):
    if clear_flag:
      return self._WrapOutput([])
    return existing_val

  def ApplyUpdateFlag(self, existing_val, update_val):
    if update_val is not None:
      output_list = self._GetPropsFieldValue(existing_val)
      update_val_list = self._GetPropsFieldValue(update_val)
      update_key_set = set([x.key for x in update_val_list])
      deduped_list = [x for x in output_list if x.key not in update_key_set]
      return self._WrapOutput(deduped_list + update_val_list)
    return existing_val

  def ApplyRemoveFlag(self, existing_val, remove_val):
    if remove_val is not None:
      output_list = self._GetPropsFieldValue(existing_val)
      remove_val_set = set(remove_val)
      return self._WrapOutput(
          [x for x in output_list if x.key not in remove_val_set])
    return existing_val




# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create ECP configurations."""


import enum
import json
import os

from googlecloudsdk.core import config
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import platforms

RESOURCE_TYPE = 'enterprise-certificate-proxy configuration file'


def get_platform_folder():
  sdk_root = config.Paths().sdk_root
  if not sdk_root:
    raise ECPConfigError(
        'Unable to find the SDK root path. The gcloud installation may be'
        ' corrupted.'
    )

  return os.path.join(sdk_root, 'platform', 'enterprise_cert')


def get_bin_folder():
  sdk_bin_path = config.Paths().sdk_bin_path
  if not sdk_bin_path:
    raise ECPConfigError(
        'Unable to find the SDK bin path. The gcloud installation may be'
        ' corrupted.'
    )

  return sdk_bin_path


def get_config_path(output_file):
  if output_file:
    return output_file
  return config.CertConfigDefaultFilePath()


def platform_to_config(platform):
  if not platform:
    platform = platforms.Platform.Current()
  if platform.operating_system == platforms.OperatingSystem.MACOSX:
    return ConfigType.KEYCHAIN
  elif platform.operating_system == platforms.OperatingSystem.LINUX:
    return ConfigType.PKCS11
  elif platform.operating_system == platforms.OperatingSystem.WINDOWS:
    return ConfigType.MYSTORE
  else:
    raise ECPConfigError(
        (
            'Unsupported platform {}. Enterprise Certificate Proxy currently'
            ' only supports OSX, Windows, and Linux.'
        ).format(platform.operating_system)
    )


class ConfigType(enum.Enum):
  PKCS11 = 1
  KEYCHAIN = 2
  MYSTORE = 3
  WORKLOAD = 4


class WindowsBinaryPathConfig(object):
  """Configuration for the paths to the ECP binaries on Windows.

  Attributes:
    ecp: Path to the ECP binary.
    ecp_http_proxy: Path to the ECP HTTP proxy binary.
    ecp_client: Path to the ECP client library.
    tls_offload: Path to the TLS offload library.
  """

  def __init__(self, ecp, ecp_client, tls_offload, ecp_http_proxy):
    self.ecp = ecp if ecp else os.path.join(get_bin_folder(), 'ecp.exe')
    self.ecp_http_proxy = (
        ecp_http_proxy
        if ecp_http_proxy
        else os.path.join(get_bin_folder(), 'ecp_http_proxy.exe')
    )
    self.ecp_client = (
        ecp_client
        if ecp_client
        else os.path.join(get_platform_folder(), 'libecp.dll')
    )
    self.tls_offload = (
        tls_offload
        if tls_offload
        else os.path.join(get_platform_folder(), 'libtls_offload.dll')
    )


class LinuxPathConfig(object):
  """Configuration for the paths to the ECP binaries on Linux.

  Attributes:
    ecp: Path to the ECP binary.
    ecp_http_proxy: Path to the ECP HTTP proxy binary.
    ecp_client: Path to the ECP client library.
    tls_offload: Path to the TLS offload library.
  """

  def __init__(self, ecp, ecp_client, tls_offload, ecp_http_proxy):
    self.ecp = ecp if ecp else os.path.join(get_bin_folder(), 'ecp')
    self.ecp_http_proxy = (
        ecp_http_proxy
        if ecp_http_proxy
        else os.path.join(get_bin_folder(), 'ecp_http_proxy')
    )
    self.ecp_client = (
        ecp_client
        if ecp_client
        else os.path.join(get_platform_folder(), 'libecp.so')
    )
    self.tls_offload = (
        tls_offload
        if tls_offload
        else os.path.join(get_platform_folder(), 'libtls_offload.so')
    )


class MacOSBinaryPathConfig(object):
  """Configuration for the paths to the ECP binaries on MacOS.

  Attributes:
    ecp: Path to the ECP binary.
    ecp_http_proxy: Path to the ECP HTTP proxy binary.
    ecp_client: Path to the ECP client library.
    tls_offload: Path to the TLS offload library.
  """

  def __init__(self, ecp, ecp_client, tls_offload, ecp_http_proxy):
    self.ecp = ecp if ecp else os.path.join(get_bin_folder(), 'ecp')
    self.ecp_http_proxy = (
        ecp_http_proxy
        if ecp_http_proxy
        else os.path.join(get_bin_folder(), 'ecp_http_proxy')
    )
    self.ecp_client = (
        ecp_client
        if ecp_client
        else os.path.join(get_platform_folder(), 'libecp.dylib')
    )
    self.tls_offload = (
        tls_offload
        if tls_offload
        else os.path.join(get_platform_folder(), 'libtls_offload.dylib')
    )


class PKCS11Config(object):

  def __init__(self, module, slot, label, user_pin):
    self.module = module
    self.slot = slot
    self.label = label

    if user_pin:
      self.user_pin = user_pin


class KeyChainConfig(object):

  def __init__(self, issuer, keychain_type):
    self.issuer = issuer
    self.keychain_type = keychain_type


class MyStoreConfig(object):

  def __init__(self, issuer, store, provider):
    self.issuer = issuer
    self.store = store
    self.provider = provider


class WorkloadConfig(object):

  def __init__(self, cert_path, key_path):
    self.cert_path = cert_path
    self.key_path = key_path


def create_linux_config(base_config, **kwargs):
  """Creates a Linux ECP Config.

  Args:
    base_config: Optional parameter to use as a fallback for parameters that are
      not set in kwargs.
    **kwargs: Linux config parameters. See go/enterprise-cert-config for valid
      variables.

  Returns:
    A dictionary object containing the ECP config.
  """
  if base_config:
    base_linux_config = base_config.get('cert_configs', {}).get('pkcs11', {})
    base_libs_config = base_config.get('libs', {})
  else:
    base_linux_config = {}
    base_libs_config = {}

  ecp_config = PKCS11Config(
      kwargs.get('module', None) or base_linux_config.get('module', None),
      kwargs.get('slot', None) or base_linux_config.get('slot', 0),
      kwargs.get('label', None) or base_linux_config.get('label', None),
      kwargs.get('user_pin', None) or base_linux_config.get('user_pin', None),
  )
  lib_config = LinuxPathConfig(
      kwargs.get('ecp', None) or base_libs_config.get('ecp', None),
      kwargs.get('ecp_client', None)
      or base_libs_config.get('ecp_client', None),
      kwargs.get('tls_offload', None)
      or base_libs_config.get('tls_offload', None),
      kwargs.get('ecp_http_proxy', None)
      or base_libs_config.get('ecp_http_proxy', None),
  )
  return {'pkcs11': vars(ecp_config)}, {'libs': vars(lib_config)}


def create_macos_config(base_config, **kwargs):
  """Creates a MacOS ECP Config.

  Args:
    base_config: Optional parameter to use as a fallback for parameters that are
      not set in kwargs.
    **kwargs: MacOS config parameters. See go/enterprise-cert-config for valid
      variables.

  Returns:
    A dictionary object containing the ECP config.
  """
  if base_config:
    base_macos_config = base_config['cert_configs']['macos_keychain']
    base_libs_config = base_config['libs']
  else:
    base_macos_config = {}
    base_libs_config = {}

  ecp_config = KeyChainConfig(
      kwargs.get('issuer', None) or base_macos_config.get('issuer', None),
      kwargs.get('keychain_type', 'all')
      or base_macos_config.get('keychain_type', 'all'),
  )
  lib_config = MacOSBinaryPathConfig(
      kwargs.get('ecp', None) or base_libs_config.get('ecp', None),
      kwargs.get('ecp_client', None)
      or base_libs_config.get('ecp_client', None),
      kwargs.get('tls_offload', None)
      or base_libs_config.get('tls_offload', None),
      kwargs.get('ecp_http_proxy', None)
      or base_libs_config.get('ecp_http_proxy', None),
  )
  return {'macos_keychain': vars(ecp_config)}, {'libs': vars(lib_config)}


def create_windows_config(base_config, **kwargs):
  """Creates a Windows ECP Config.

  Args:
    base_config: Optional parameter to use as a fallback for parameters that are
      not set in kwargs.
    **kwargs: Windows config parameters. See go/enterprise-cert-config for valid
      variables.

  Returns:
    A dictionary object containing the ECP config.
  """
  if base_config:
    base_windows_config = base_config['cert_configs']['windows_store']
    base_libs_config = base_config['libs']
  else:
    base_windows_config = {}
    base_libs_config = {}

  ecp_config = MyStoreConfig(
      kwargs.get('issuer', None) or base_windows_config.get('issuer', None),
      kwargs.get('store', None) or base_windows_config.get('store', None),
      kwargs.get('provider', None) or base_windows_config.get('provider', None),
  )
  lib_config = WindowsBinaryPathConfig(
      kwargs.get('ecp', None) or base_libs_config.get('ecp', None),
      kwargs.get('ecp_client', None)
      or base_libs_config.get('ecp_client', None),
      kwargs.get('tls_offload', None)
      or base_libs_config.get('tls_offload', None),
      kwargs.get('ecp_http_proxy', None)
      or base_libs_config.get('ecp_http_proxy', None),
  )
  return {'windows_store': vars(ecp_config)}, {'libs': vars(lib_config)}


def create_workload_config(base_config, **kwargs):
  """Creates a Workload ECP Config.

  Args:
    base_config: Optional parameter to use as a fallback for parameters that are
      not set in kwargs.
    **kwargs: Workload config parameters. See go/enterprise-cert-config for
      valid variables.

  Returns:
    A dictionary object containing the ECP config.
  """
  if base_config:
    base_workload_config = base_config['cert_configs']['workload']
  else:
    base_workload_config = {}

  workload_config = WorkloadConfig(
      kwargs.get('cert_path', None)
      or base_workload_config.get('cert_path', None),
      kwargs.get('key_path', None)
      or base_workload_config.get('key_path', None),
  )

  return {'workload': vars(workload_config)}, {}


def create_ecp_config(config_type, base_config=None, **kwargs):
  """Creates an ECP Config.

  Args:
    config_type: An ConfigType Enum that describes the type of ECP config.
    base_config: Optional parameter to use as a fallback for parameters that are
      not set in kwargs.
    **kwargs: config parameters. See go/enterprise-cert-config for valid
      variables.

  Returns:
    A dictionary object containing the ECP config.
  Raises:
    ECPConfigError: No valid config_type is specified.
  """

  if config_type == ConfigType.PKCS11:
    ecp_config, libs_config = create_linux_config(base_config, **kwargs)
  elif config_type == ConfigType.KEYCHAIN:
    ecp_config, libs_config = create_macos_config(base_config, **kwargs)
  elif config_type == ConfigType.MYSTORE:
    ecp_config, libs_config = create_windows_config(base_config, **kwargs)
  elif config_type == ConfigType.WORKLOAD:
    ecp_config, libs_config = create_workload_config(base_config, **kwargs)
  else:
    raise ECPConfigError(
        (
            'Unknown config_type {} passed to create enterprise certificate'
            ' configuration. Valid options are: [PKCS11, KEYCHAIN, MYSTORE]'
        ).format(config_type)
    )

  # TODO(b/459858373): remove gating for ECP HTTP Proxy on internal user check.
  if (
      not (
          properties.VALUES.context_aware.use_ecp_http_proxy.GetBool()
          and properties.IsInternalUserCheck()
      )
      and 'libs' in libs_config
      and 'ecp_http_proxy' in libs_config['libs']
  ):
    del libs_config['libs']['ecp_http_proxy']

  return {'cert_configs': ecp_config, **libs_config}


def create_config(config_type, **kwargs):
  """Creates the ECP config based on the passed in CLI arguments."""
  output = create_ecp_config(config_type, None, **kwargs)
  config_path = get_config_path(kwargs.get('output_file', None))

  files.WriteFileContents(config_path, json.dumps(output, indent=2))
  log.CreatedResource(config_path, RESOURCE_TYPE)


def update_config(config_type, **kwargs):
  """Updates the ECP config based on the passed in CLI arguments.

  Args:
    config_type: An ConfigType Enum that describes the type of ECP config.
    **kwargs: config parameters that will be updated. See
      go/enterprise-cert-config for valid variables.

  Only explicit args will overwrite existing values
  """
  config_path = get_config_path(kwargs.get('output_file', None))
  data = files.ReadFileContents(config_path)

  active_config = json.loads(data)
  output = create_ecp_config(config_type, active_config, **kwargs)

  files.WriteFileContents(config_path, json.dumps(output, indent=2))
  log.CreatedResource(config_path, RESOURCE_TYPE)


class ECPConfigError(Exception):

  def __init__(self, message):
    super(ECPConfigError, self).__init__()
    self.message = message



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Apphub Application Services API."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.apphub import consts as api_lib_consts
from googlecloudsdk.api_lib.apphub import utils as api_lib_utils
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import log


class ServicesClient(object):
  """Client for Application services in apphub API."""

  def __init__(self, release_track=base.ReleaseTrack.ALPHA):
    self.client = api_lib_utils.GetClientInstance(release_track)
    self.messages = api_lib_utils.GetMessagesModule(release_track)
    self._app_services_client = (
        self.client.projects_locations_applications_services
    )
    self._poller = waiter.CloudOperationPoller(
        self._app_services_client,
        self.client.projects_locations_operations,
    )
    self._delete_poller = waiter.CloudOperationPollerNoResources(
        self.client.projects_locations_operations,
    )

  def Describe(self, service):
    """Describe a Application Service in the Project/location.

    Args:
      service: str, the name for the service being described.

    Returns:
      Described service Resource.
    """
    describe_req = (
        self.messages.ApphubProjectsLocationsApplicationsServicesGetRequest(
            name=service
        )
    )
    return self._app_services_client.Get(describe_req)

  def List(
      self,
      parent,
      limit=None,
      page_size=100,
  ):
    """List application services in the Projects/Location.

    Args:
      parent: str,
        projects/{projectId}/locations/{location}/applications/{application}
      limit: int or None, the total number of results to return. Default value
        is None
      page_size: int, the number of entries in each batch (affects requests
        made, but not the yielded results). Default value is 100.

    Returns:
      Generator of matching application services.
    """
    list_req = (
        self.messages.ApphubProjectsLocationsApplicationsServicesListRequest(
            parent=parent
        )
    )
    return list_pager.YieldFromList(
        self._app_services_client,
        list_req,
        field='services',
        batch_size=page_size,
        limit=limit,
        batch_size_attribute='pageSize',
    )

  def Delete(self, service, async_flag):
    """Delete a application service in the Project/location.

    Args:
      service: str, the name for the service being deleted
      async_flag: Boolean value for async operation. If true the operation will
        be async

    Returns:
      Empty Response Message or Operation based on async flag value.
    """
    delete_req = (
        self.messages.ApphubProjectsLocationsApplicationsServicesDeleteRequest(
            name=service
        )
    )
    operation = self._app_services_client.Delete(delete_req)

    if async_flag:
      return operation

    delete_response = api_lib_utils.WaitForOperation(
        self._delete_poller,
        operation,
        api_lib_consts.DeleteApplicationService.WAIT_FOR_DELETE_MESSAGE,
        api_lib_consts.DeleteApplicationService.DELETE_TIMELIMIT_SEC,
    )

    return delete_response

  def Create(
      self,
      service_id,
      parent,
      async_flag,
      discovered_service,
      display_name,
      description,
      attributes,
  ):
    """Creates an application in the Project/location.

    Args:
      service_id: str, Service ID
      parent: parent for Application resource
      async_flag: Boolean value for async operation. If true the operation will
        be async
      discovered_service: str, Discovered service name
      display_name: str, Human-friendly display name
      description: str, Description of the Service
      attributes: Attributes, Attributes of the Service

    Returns:
      Service or Operation based on async flag value.
    """
    service = self.messages.Service(
        description=description,
        displayName=display_name,
        discoveredService=discovered_service,
        attributes=attributes,
    )

    create_req = (
        self.messages.ApphubProjectsLocationsApplicationsServicesCreateRequest(
            service=service, serviceId=service_id, parent=parent
        )
    )
    operation = self._app_services_client.Create(create_req)

    if async_flag:
      return operation

    create_response = api_lib_utils.WaitForOperation(
        self._poller,
        operation,
        api_lib_consts.CreateApplicationService.WAIT_FOR_CREATE_MESSAGE,
        api_lib_consts.CreateApplicationService.CREATE_TIMELIMIT_SEC,
    )

    return create_response

  def Update(self, service_id, async_flag, attributes, args):
    """Update application service."""
    service, update_mask = self._UpdateHelper(args, attributes)

    if not update_mask:
      log.status.Print(
          api_lib_consts.UpdateApplicationService.EMPTY_UPDATE_HELP_TEXT
      )
      return

    update_request = (
        self.messages.ApphubProjectsLocationsApplicationsServicesPatchRequest(
            name=service_id,
            service=service,
            updateMask=update_mask,
        )
    )

    operation = self._app_services_client.Patch(update_request)

    if async_flag:
      return operation
    update_response = api_lib_utils.WaitForOperation(
        self._poller,
        operation,
        api_lib_consts.UpdateApplicationService.WAIT_FOR_UPDATE_MESSAGE,
        api_lib_consts.UpdateApplicationService.UPDATE_TIMELIMIT_SEC,
    )

    return update_response

  def _UpdateHelper(self, args, attributes):
    """Helper to generate service and update_mask fields for update_request."""
    service = self.messages.Service()
    update_mask = ''

    if args.display_name:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_DISPLAY_NAME_FIELD_NAME,
      )
      service.displayName = args.display_name

    if args.description:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_DESCRIPTION_FIELD_NAME,
      )
      service.description = args.description

    if attributes.criticality:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_ATTR_CRITICALITY_FIELD_NAME,
      )
    if attributes.environment:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_ATTR_ENVIRONMENT_FIELD_NAME,
      )
    if attributes.businessOwners:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_ATTR_BUSINESS_OWNERS_FIELD_NAME,
      )
    if attributes.developerOwners:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_ATTR_DEVELOPER_OWNERS_FIELD_NAME,
      )
    if attributes.operatorOwners:
      update_mask = api_lib_utils.AddToUpdateMask(
          update_mask,
          api_lib_consts.UpdateApplicationService.UPDATE_MASK_ATTR_OPERATOR_OWNERS_FIELD_NAME,
      )

    service.attributes = attributes

    return service, update_mask



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementation of list command for Insights dataset config."""

import re

from googlecloudsdk.api_lib.storage import insights_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import flags

LOCATION_REGEX_PATTERN = re.compile(r'locations/(.*)/.*/')


def _transform_location(dataset_config):
  matched_result = re.search(LOCATION_REGEX_PATTERN, dataset_config['name'])
  if matched_result and matched_result.group(1) is not None:
    return matched_result.group(1)
  else:
    return 'N/A-Misformated Value'


_TRANSFORMS = {'location_transform': _transform_location}


@base.DefaultUniverseOnly
class List(base.ListCommand):
  """List returns all the Insights dataset configs for given location."""

  hints = base.CommandHint(read_only=True)

  detailed_help = {
      'DESCRIPTION': """
      List Cloud storage Insights dataset configs.
      """,
      'EXAMPLES': """

      List all dataset configs in all locations:

          $ {command}

      List all dataset configs for location "us-central1":

          $ {command} --location=us-central1

      List all dataset configs with a page size of "20":

          $ {command} --location=us-central1 --page-size=20

      List all dataset configs with JSON formatting:

          $ {command} --location=us-central1 --format=json
      """,
  }

  @staticmethod
  def Args(parser):
    flags.add_dataset_config_location_flag(parser, is_required=False)
    parser.display_info.AddFormat("""
        table(
            uid:label=DATASET_CONFIG_ID,
            name.basename():label=DATASET_CONFIG_NAME,
            location_transform():label=LOCATION,
            sourceProjects.projectNumbers:label=SOURCE_PROJECTS,
            organizationNumber:label=ORGANIZATION_NUMBER,
            retentionPeriodDays:label=RETENTION_PERIOD_DAYS,
            datasetConfigState:label=STATE
        )
        """)
    parser.display_info.AddTransforms(_TRANSFORMS)

  def Run(self, args):
    return insights_api.InsightsApi().list_dataset_config(
        location=args.location, page_size=args.page_size
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to set an IAM policy on a Data Fusion instance or a namespace."""


from googlecloudsdk.api_lib.data_fusion import datafusion as df
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.data_fusion import data_fusion_iam_util
from googlecloudsdk.command_lib.data_fusion import resource_args
from googlecloudsdk.command_lib.iam import iam_util


def SetIamPolicyFromFile(instance_ref,
                         namespace,
                         policy_file,
                         messages,
                         client):
  """Reads an instance's IAM policy from a file, and sets it."""
  new_iam_policy = data_fusion_iam_util.ParsePolicyFile(
      policy_file,
      messages.Policy)

  return data_fusion_iam_util.DoSetIamPolicy(
      instance_ref, namespace, new_iam_policy, messages, client)


@base.RegionalEndpointsSupported
class SetIamPolicy(base.Command):
  r"""Sets the IAM policy for a Cloud Data Fusion instance.

  ## EXAMPLES

  To set the policy for instance `my-instance` in project `my-project`, location
  in `my-location`, and zone in `my-zone` run:

  $ {command} my-instance policy-file.yaml --project=my-project \
    --location=my-location

  To do the same in a particular namespace, run:
  $ {command} my-instance policy-file.yaml --project=my-project \
    --location=my-location [--namespace=NAMESPACE]
  """

  @staticmethod
  def Args(parser):
    resource_args.AddInstanceResourceArg(parser, 'Instance to set.')
    base.URI_FLAG.RemoveFromParser(parser)
    iam_util.AddArgForPolicyFile(parser)
    parser.add_argument(
        '--namespace',
        help='CDAP Namespace whose IAM policy we wish to set. '
        'For example: `--namespace=my-namespace`.')

  def Run(self, args):
    instance_ref = args.CONCEPTS.instance.Parse()
    datafusion = df.Datafusion(location=instance_ref.locationsId)

    results = SetIamPolicyFromFile(instance_ref, args.namespace,
                                   args.policy_file, datafusion.messages,
                                   datafusion.client)
    return results



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update a subordinate certificate authority."""


from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import operations
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.command_lib.privateca import update_utils
from googlecloudsdk.command_lib.util.args import labels_util


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.UpdateCommand):
  r"""Update an existing subordinate certificate authority.

  ## EXAMPLES
    To update labels on a subordinate CA:

      $ {command} server-tls-1 \
        --pool=my-pool \
        --location=us-west1 \
        --update-labels=foo=bar

    To update the CA certificate chain for a subordinate CA:

      $ {command} server-tls-1 \
        --pool=my-pool \
        --location=us-west1 \
        --pem-chain=pem_chain.txt
  """

  @staticmethod
  def Args(parser):
    resource_args.AddCertAuthorityPositionalResourceArg(parser, 'to update')
    base.Argument(
        '--pem-chain',
        required=False,
        help=(
            'A file containing a list of PEM-encoded certificates that'
            ' represent the issuing chain of this CA. Please note'
            ' that the certificate corresponding to this specific CA should be'
            ' excluded.'
        ),
    ).AddToParser(parser)
    labels_util.AddUpdateLabelsFlags(parser)

  def Run(self, args):
    client = privateca_base.GetClientInstance(api_version='v1')
    messages = privateca_base.GetMessagesModule(api_version='v1')

    ca_ref = args.CONCEPTS.certificate_authority.Parse()
    ca_name = ca_ref.RelativeName()

    current_ca = client.projects_locations_caPools_certificateAuthorities.Get(
        messages.PrivatecaProjectsLocationsCaPoolsCertificateAuthoritiesGetRequest(
            name=ca_name
        )
    )

    resource_args.CheckExpectedCAType(
        messages.CertificateAuthority.TypeValueValuesEnum.SUBORDINATE,
        current_ca,
        version='v1',
    )

    ca_to_update, update_mask = update_utils.UpdateCAFromArgs(
        args, current_ca.labels
    )

    # Patch is the gcloud client lib method to update a CA.
    operation = client.projects_locations_caPools_certificateAuthorities.Patch(
        messages.PrivatecaProjectsLocationsCaPoolsCertificateAuthoritiesPatchRequest(
            name=ca_name,
            certificateAuthority=ca_to_update,
            updateMask=','.join(update_mask),
            requestId=request_utils.GenerateRequestId(),
        )
    )

    return operations.Await(
        operation, 'Updating Subordinate CA.', api_version='v1'
    )
